@@ -31,6 +31,60 @@
 
 #include "afterconf.h"
 
+TermDef       BalloonContentsTerms[] = {
+    ASCF_DEFINE_KEYWORD_F(BALLOON, TF_NO_MYNAME_PREPENDING, Name		, NULL, BALLOON_SHOW_Name, ),
+    ASCF_DEFINE_KEYWORD_F(BALLOON, TF_NO_MYNAME_PREPENDING, IconName	, NULL, BALLOON_SHOW_IconName, ),
+    ASCF_DEFINE_KEYWORD_F(BALLOON, TF_NO_MYNAME_PREPENDING, ResClass	, NULL, BALLOON_SHOW_ResClass, ),
+    ASCF_DEFINE_KEYWORD_F(BALLOON, TF_NO_MYNAME_PREPENDING, ResName		, NULL, BALLOON_SHOW_ResName, ),
+    ASCF_DEFINE_KEYWORD_F(BALLOON, TF_NO_MYNAME_PREPENDING, Comment		, NULL, BALLOON_SHOW_Comment, ),
+    ASCF_DEFINE_KEYWORD_F(BALLOON, TF_NO_MYNAME_PREPENDING, Exec		, NULL, BALLOON_SHOW_Exec, ),
+    ASCF_DEFINE_KEYWORD_F(BALLOON, TF_NO_MYNAME_PREPENDING, GenericName	, NULL, BALLOON_SHOW_GenericName, ),
+    {0, NULL, 0, 0, 0}
+};
+
+SyntaxDef     BalloonContentsSyntax = {
+	',',
+	'\n',
+    BalloonContentsTerms,
+	0,										   /* use default hash size */
+    ' ',
+	" ",
+	"\t",
+    "Balloon Contents flags",
+	"Balloon Contents",
+	"",
+	NULL,
+	0
+};
+struct SyntaxDef     *BalloonContentsSyntaxPtr = &BalloonContentsSyntax;
+
+flag_options_xref BalloonContentsFlagsXref[] = {
+    {BALLOON_SHOW_Name, 		BALLOON_Name_ID			, 0},
+    {BALLOON_SHOW_IconName, 	BALLOON_IconName_ID		, 0},
+    {BALLOON_SHOW_ResClass, 	BALLOON_ResClass_ID		, 0},
+    {BALLOON_SHOW_ResName, 		BALLOON_ResName_ID		, 0},
+    {BALLOON_SHOW_Comment, 		BALLOON_Comment_ID		, 0},
+    {BALLOON_SHOW_Exec, 		BALLOON_Exec_ID			, 0},
+    {BALLOON_SHOW_GenericName, 	BALLOON_GenericName_ID	, 0},
+    {0, 0, 0}
+};
+
+ASFlagType
+ParseBalloonContentsOptions( FreeStorageElem * options )
+{
+    ASFlagType hints = 0 ;
+    while( options )
+	{
+        LOCAL_DEBUG_OUT( "options(%p)->keyword(\"%s\")", options, options->term->keyword );
+        if (options->term != NULL)
+            ReadFlagItem (NULL, &hints, options, BalloonContentsFlagsXref);
+        options = options->next;
+    }
+    return hints;
+}
+
+
+int ASDefaultBalloonTypes[]= { BALLOON_ID_START, 0 };
 
 /*****************************************************************************
  *
@@ -43,18 +97,21 @@ balloonConfig *
 Create_balloonConfig ()
 {
   balloonConfig *config = (balloonConfig *) safecalloc (1, sizeof (balloonConfig));
+  config->type = CONFIG_Balloons_ID ; 
+  
   return config;
 }
 
 void
 Destroy_balloonConfig (balloonConfig * config)
 {
-  if (!config)
-	return;
-  if (config->Style)
-    free (config->Style);
-  free (config);
-  config = NULL;
+  	if (!config)
+		return;
+  	if (config->Style)
+    	free (config->Style);
+	config->type = 0 ; 
+  	free (config);
+  	config = NULL;
 }
 
 void
@@ -64,7 +121,8 @@ Print_balloonConfig (balloonConfig *config )
         fprintf( stderr, "No balloon configuration available \n");
     else
     {
-        fprintf( stderr,"set_flags = 0x%lX\n",config->set_flags);
+        fprintf( stderr,"BALLOON.set_flags = 0x%lX\n",config->set_flags);
+        fprintf( stderr,"BALLOON.flags = 0x%lX\n",config->flags);
 		ASCF_PRINT_FLAGS_KEYWORD(stderr,BALLOON,config,BorderHilite);
 		ASCF_PRINT_INT_KEYWORD(stderr,BALLOON,config,XOffset);
 		ASCF_PRINT_INT_KEYWORD(stderr,BALLOON,config,YOffset);
@@ -77,38 +135,40 @@ Print_balloonConfig (balloonConfig *config )
 }
 
 void
-balloon_config2look( MyLook *look, balloonConfig *config, const char *default_style )
+balloon_config2look( MyLook *look, ASBalloonLook *balloon_look, balloonConfig *config, const char *default_style )
 {
     if( look )
     {
-        if( look->balloon_look == NULL )
-            look->balloon_look = safecalloc( 1, sizeof(ASBalloonLook) );
-
+		if( balloon_look == NULL ) 
+		{
+        	if( look->balloon_look == NULL )
+            	look->balloon_look = create_balloon_look();
+			balloon_look = look->balloon_look ;
+		}
         if( config == NULL )
 		{	
-            memset( look->balloon_look, 0x00, sizeof(ASBalloonLook) );
-			look->balloon_look->show = True ;
-        	look->balloon_look->XOffset = 5 ;
-        	look->balloon_look->YOffset = 5 ;
-			look->balloon_look->Style = mystyle_list_find_or_default (look->styles_list, default_style);
-			look->balloon_look->Delay = 200 ;
-			look->balloon_look->CloseDelay = 2000 ;
+			balloon_look->show = True ;
+        	balloon_look->XOffset = 5 ;
+        	balloon_look->YOffset = 5 ;
+			balloon_look->Style = mystyle_list_find_or_default (look->styles_list, default_style);
+			balloon_look->Delay = 200 ;
+			balloon_look->CloseDelay = 2000 ;
 		}else
         {
-#define MERGE_BALLOON_SCALAR_VAL(val)  look->balloon_look->val = config->val 			
+#define MERGE_BALLOON_SCALAR_VAL(val)  balloon_look->val = config->val 			
 
-            look->balloon_look->show = get_flags( config->set_flags, BALLOON_USED );
+            balloon_look->show = get_flags( config->flags, BALLOON_USED );
 			MERGE_BALLOON_SCALAR_VAL(BorderHilite);
 			MERGE_BALLOON_SCALAR_VAL(XOffset);
 			MERGE_BALLOON_SCALAR_VAL(YOffset);
 			MERGE_BALLOON_SCALAR_VAL(Delay);
 			MERGE_BALLOON_SCALAR_VAL(CloseDelay);
-            look->balloon_look->Style = mystyle_list_find_or_default (look->styles_list, config->Style?config->Style:default_style);
+            balloon_look->Style = mystyle_list_find_or_default (look->styles_list, config->Style?config->Style:default_style);
 			MERGE_BALLOON_SCALAR_VAL(TextPaddingX);
 			MERGE_BALLOON_SCALAR_VAL(TextPaddingY);
 		}
-	    LOCAL_DEBUG_OUT( "balloon mystyle = %p (\"%s\")", look->balloon_look->Style,
-                    	 look->balloon_look->Style?look->balloon_look->Style->name:"none" );
+	    LOCAL_DEBUG_OUT( "balloon mystyle = %p (\"%s\")", balloon_look->Style,
+                    	 balloon_look->Style?balloon_look->Style->name:"none" );
 
     }
 }
@@ -123,9 +183,14 @@ set_default_balloon_style( balloonConfig *config, const char *style )
     }
 }
 
+flag_options_xref BalloonsFlags[] = {
+	ASCF_DEFINE_MODULE_ONOFF_FLAG_XREF(BALLOON,Balloons,balloonConfig),
+    {0, 0, 0}
+};
+
 
 balloonConfig*
-Process_balloonOptions (FreeStorageElem * options, balloonConfig *config)
+Process_balloonOptions (FreeStorageElem * options, balloonConfig *config, int id_base )
 {
     ConfigItem item;
     item.memory = NULL;
@@ -135,25 +200,32 @@ Process_balloonOptions (FreeStorageElem * options, balloonConfig *config)
 
     for (; options; options = options->next)
     {
-        if (options->term->id < BALLOON_ID_START || options->term->id > BALLOON_ID_END )
+		int orig_id = options->term->id ; 
+		int id = (int)options->term->id - id_base + BALLOON_ID_START ; 
+        if (id < BALLOON_ID_START || id > BALLOON_ID_END )
             continue;
 
         if (options->term == NULL)
             continue;
         if (options->term->type == TT_FLAG)
         {
-            switch(options->term->id)
+			options->term->id = id ; 
+			if( !ReadFlagItemAuto (config, 0, options, &BalloonsFlags[0]) )
 			{
-				case BALLOON_USED_ID :  config->set_flags |= BALLOON_USED ; break;
-	            ASCF_HANDLE_BEVEL_KEYWORD_CASE(BALLOON,config,options,BorderHilite); 
-            }
+	            switch(id )
+				{
+		            ASCF_HANDLE_BEVEL_KEYWORD_CASE(BALLOON,config,options,BorderHilite); 
+				}
+			}
+			LOCAL_DEBUG_OUT( "%s - %lX:%lX", options->term->keyword, config->set_flags, config->flags );
+			options->term->id = orig_id ;
             continue;
         }
 
         if (!ReadConfigItem (&item, options))
             continue;
 
-        switch (options->term->id)
+        switch (id)
         {
 			ASCF_HANDLE_INTEGER_KEYWORD_CASE(BALLOON,config,item,XOffset); 
 			ASCF_HANDLE_INTEGER_KEYWORD_CASE(BALLOON,config,item,YOffset); 
@@ -170,6 +242,36 @@ Process_balloonOptions (FreeStorageElem * options, balloonConfig *config)
     return config;
 }
 
+void
+MergeBalloonOptions ( ASModuleConfig *asm_to, ASModuleConfig *asm_from)
+{
+	if( asm_to && asm_from ) 
+	{
+		int i ;
+		for( i = 0 ; asm_to->class->balloon_types[i] > 0 && asm_from->class->balloon_types[i] > 0 ; ++i ) 
+		{
+			balloonConfig *from = asm_from->balloon_configs[i];
+			balloonConfig *to = asm_to->balloon_configs[i];		
+			if( to == NULL )
+	   			to = Create_balloonConfig ();
+
+			if( from != NULL ) 
+			{
+		    	ASCF_MERGE_FLAGS(to,from);
+				ASCF_MERGE_SCALAR_KEYWORD(BALLOON, to, from, BorderHilite);
+				ASCF_MERGE_SCALAR_KEYWORD(BALLOON, to, from, XOffset); 
+				ASCF_MERGE_SCALAR_KEYWORD(BALLOON, to, from, YOffset); 
+				ASCF_MERGE_SCALAR_KEYWORD(BALLOON, to, from, Delay); 
+				ASCF_MERGE_SCALAR_KEYWORD(BALLOON, to, from, CloseDelay); 
+				ASCF_MERGE_STRING_KEYWORD(BALLOON, to, from, Style);
+				ASCF_MERGE_SCALAR_KEYWORD(BALLOON, to, from, TextPaddingX); 
+				ASCF_MERGE_SCALAR_KEYWORD(BALLOON, to, from, TextPaddingY); 
+			}
+			asm_to->balloon_configs[i] = to ;
+		}
+	}
+}
+
 #if 0
 
 /* returns:
