@@ -69,6 +69,8 @@
 
 ASStorage *_as_default_storage = NULL ;
 
+#define get_default_asstorage()   (_as_default_storage?_as_default_storage:(_as_default_storage=create_asstorage()))
+
 
 /************************************************************************/
 /* Private Functions : 													*/
@@ -128,21 +130,21 @@ rlediff_compress_bitmap32( CARD8 *buffer,  CARD8* data, int size, CARD32 bitmap_
 
 
 static void
-compute_diff8( register short *diff, register CARD8 *data, int size ) 
+compute_diff8( register ASStorageDiff *diff, register CARD8 *data, int size ) 
 {
 	register int i = 0;	
 	diff[0] = data[0] ;
 /*	fprintf( stderr, "%d(%4.4X) ", diff[0], diff[0] ); */
 	while( ++i < size ) 
 	{	
-		diff[i] = (short)data[i] - (short)data[i-1] ;
+		diff[i] = (ASStorageDiff)data[i] - (ASStorageDiff)data[i-1] ;
 /*		fprintf( stderr, "%d(%4.4X) ", diff[i], diff[i] ); */
 	}
 /*	fprintf( stderr, "\n" ); */
 }	   
 
 static void
-compute_diff32( register short *diff, CARD8 *data, int size ) 
+compute_diff32( register ASStorageDiff *diff, CARD8 *data, int size ) 
 {
 	register int i = 0;	
 	register CARD32 *data32 = (CARD32*)data ;
@@ -150,25 +152,104 @@ compute_diff32( register short *diff, CARD8 *data, int size )
 /*	fprintf( stderr, "\n0:%d(%4.4X) ", diff[0], diff[0] ); */
 	while( ++i < size ) 
 	{	
-		diff[i] = (short)data32[i] - (short)data32[i-1] ;
+		diff[i] = (ASStorageDiff)data32[i] - (ASStorageDiff)data32[i-1] ;
 /*		fprintf( stderr, "%d:%d(%4.4X) ", i, diff[i], diff[i] ); */
 	}
 /*	fprintf( stderr, "\n" ); */
 }	   
 
 static void
-compute_diff32_8bitshift( register short *diff, CARD8 *data, int size ) 
+compute_diff32_8bitshift( register ASStorageDiff *diff, CARD8 *data, int size ) 
 {
 	register int i = 0;	
 	register CARD32 *data32 = (CARD32*)data ;
-	diff[0] = data32[0]>>8 ;
+	register ASStorageDiff dp = data32[0]>>8;
+	diff[0] = dp ;
 	while( ++i < size ) 
-		diff[i] = (short)(data32[i]>>8) - (short)(data32[i-1]>>8) ;
+	{
+		register ASStorageDiff d = data32[i]>>8;
+		diff[i] = d - dp ;
+		dp = d;
+	}
 }	   
 
+static void
+compute_diff32_16bitshift( register ASStorageDiff *diff, CARD8 *data, int size ) 
+{
+	register int i = 0;	
+	register CARD32 *data32 = (CARD32*)data ;
+	register ASStorageDiff dp = data32[0]>>16;
+	diff[0] = dp ;
+	while( ++i < size ) 
+	{
+		register ASStorageDiff d = data32[i]>>16;
+		diff[i] = d - dp ;
+		dp = d;
+	}
+}	   
+
+static void
+compute_diff32_masked( register ASStorageDiff *diff, CARD8 *data, int size ) 
+{
+	register int i = 0;	
+	register CARD32 *data32 = (CARD32*)data ;
+	register ASStorageDiff dp = data32[0]&0x0ff;
+	diff[0] = dp ;
+	while( ++i < size ) 
+	{
+		register ASStorageDiff d = data32[i]&0x0ff;
+		diff[i] = d - dp ;
+		dp = d;
+	}
+}	   
+
+static void
+compute_diff32_8bitshift_masked( register ASStorageDiff *diff, CARD8 *data, int size ) 
+{
+	register int i = 0;	
+	register CARD32 *data32 = (CARD32*)data ;
+	register ASStorageDiff dp = (data32[0]>>8)&0x0ff;
+	diff[0] = dp ;
+	while( ++i < size ) 
+	{
+		register ASStorageDiff d = (data32[i]>>8)&0x0ff;
+		diff[i] = d - dp ;
+		dp = d;
+	}
+}	   
+
+static void
+compute_diff32_16bitshift_masked( register ASStorageDiff *diff, CARD8 *data, int size ) 
+{
+	register int i = 0;	
+	register CARD32 *data32 = (CARD32*)data ;
+	register ASStorageDiff dp = (data32[0]>>16)&0x0ff;
+	diff[0] = dp ;
+	while( ++i < size ) 
+	{
+		register ASStorageDiff d = (data32[i]>>16)&0x0ff;
+		diff[i] = d - dp ;
+		dp = d;
+	}
+}	   
+
+static void
+compute_diff32_24bitshift_masked( register ASStorageDiff *diff, CARD8 *data, int size ) 
+{
+	register int i = 0;	
+	register CARD32 *data32 = (CARD32*)data ;
+	register ASStorageDiff dp = (data32[0]>>24)&0x0ff;
+	diff[0] = dp ;
+	while( ++i < size ) 
+	{
+		register ASStorageDiff d = (data32[i]>>24)&0x0ff;
+		diff[i] = d - dp ;
+		dp = d;
+	}
+}	   
 
 static int 
-rlediff_compress( CARD8 *buffer,  short *diff, int size )
+rlediff_compress( CARD8 *buffer,  ASStorageDiff *diff, int size )
 {
 	int comp_size = 1 ;
 	int i = 1;
@@ -512,15 +593,175 @@ rlediff_decompress( CARD8 *buffer,  CARD8* data, int size )
 }	 
 
 
+static int
+copy_data_tinted (CARD8 *buffer, CARD32 *data32, int size, CARD32 tint)
+{
+	int comp_size = 0;
+	do{	buffer[comp_size] = (data32[comp_size]*tint)>>8;}while (++comp_size < size);
+	return comp_size;
+}
+
+static int
+copy_data_tinted_8bitshift (CARD8 *buffer, CARD32 *data32, int size, CARD32 tint)
+{
+	int comp_size = 0;
+	do{	buffer[comp_size] = (data32[comp_size]*tint)>>16;}while (++comp_size < size);
+	return comp_size;
+}
+
+static int
+copy_data_tinted_16bitshift (CARD8 *buffer, CARD32 *data32, int size, CARD32 tint)
+{
+	int comp_size = 0;
+	do{	buffer[comp_size] = (data32[comp_size]*tint)>>24;}while (++comp_size < size);
+	return comp_size;
+}
+
+static int
+copy_data_tinted_masked ( CARD8 *buffer, CARD32 *data32, int size, CARD32 tint)
+{
+	int comp_size = 0;
+	do{	buffer[comp_size] = ((data32[comp_size]&0x0FF)*tint)>>8;}while (++comp_size < size);
+	return comp_size;
+}
+
+static int
+copy_data_tinted_8bitshift_masked (CARD8 *buffer, CARD32 *data32, int size, CARD32 tint)
+{
+	int comp_size = 0;
+	do{	buffer[comp_size] = (((data32[comp_size]>>8)&0x0FF)*tint)>>8;}while (++comp_size < size);
+	return comp_size;
+}
+
+
+static int
+copy_data_tinted_16bitshift_masked (CARD8 *buffer, CARD32 *data32, int size, CARD32 tint)
+{
+	int comp_size = 0;
+	do{	buffer[comp_size] = (((data32[comp_size]>>16)&0x0FF)*tint)>>8;}while (++comp_size < size);
+	return comp_size;
+}
+
+static int
+copy_data_tinted_24bitshift_masked( CARD8 *buffer, CARD32 *data32, int size, CARD32 tint)
+{
+	int comp_size = 0;
+	do{	buffer[comp_size] = (((data32[comp_size]>>24)&0x0FF)*tint)>>8;}while (++comp_size < size);
+	return comp_size;
+}
+
+static int
+copy_data32 (CARD8 *buffer, CARD32 *data32, int size)
+{
+	int comp_size = 0;
+	do{	buffer[comp_size] = data32[comp_size];}while (++comp_size < size);
+	return comp_size;
+}
+
+static int
+copy_data32_8bitshift (CARD8 *buffer, CARD32 *data32, int size)
+{
+	int comp_size = 0;
+	do{	buffer[comp_size] = data32[comp_size]>>8;}while (++comp_size < size);
+	return comp_size;
+}
+
+static int
+copy_data32_16bitshift (CARD8 *buffer, CARD32 *data32, int size)
+{
+	int comp_size = 0;
+	do{	buffer[comp_size] = data32[comp_size]>>16;}while (++comp_size < size);
+	return comp_size;
+}
+
+static int
+copy_data32_masked ( CARD8 *buffer, CARD32 *data32, int size)
+{
+	int comp_size = 0;
+	do{	buffer[comp_size] = data32[comp_size]&0x0FF;}while (++comp_size < size);
+	return comp_size;
+}
+
+static int
+copy_data32_8bitshift_masked (CARD8 *buffer, CARD32 *data32, int size)
+{
+	int comp_size = 0;
+	do{	buffer[comp_size] = (data32[comp_size]>>8)&0x0FF;}while (++comp_size < size);
+	return comp_size;
+}
+
+
+static int
+copy_data32_16bitshift_masked (CARD8 *buffer, CARD32 *data32, int size)
+{
+	int comp_size = 0;
+	do{	buffer[comp_size] = (data32[comp_size]>>16)&0x0FF;}while (++comp_size < size);
+	return comp_size;
+}
+
+static int
+copy_data32_24bitshift_masked( CARD8 *buffer, CARD32 *data32, int size)
+{
+	int comp_size = 0;
+	do{	buffer[comp_size] = (data32[comp_size]>>24)&0x0FF;}while (++comp_size < size);
+	return comp_size;
+}
+
+
 static CARD8* 
-compress_stored_data( ASStorage *storage, CARD8 *data, int size, ASFlagType *flags, int *compressed_size, 
+compress_stored_data( ASStorage *storage, CARD8 *data, int size, ASFlagType *flags, int *compressed_size,
 					  CARD32 bitmap_threshold )
 {
 	/* TODO: just a stub for now - need to implement compression */
 	int comp_size = size ;
 	CARD8  *buffer = data ;
 	size_t 	buf_size = size ; 
-	
+
+	static compute_diff_func_type compute_diff_func[2][4] = 
+	{	{
+			compute_diff32,
+			compute_diff32_8bitshift,
+			compute_diff32_16bitshift,
+			compute_diff32_24bitshift_masked /* to clear the sign bit ! */
+		},
+		{
+			compute_diff32_masked,
+			compute_diff32_8bitshift_masked,
+			compute_diff32_16bitshift_masked,
+			compute_diff32_24bitshift_masked
+		}
+	};
+
+	static copy_data32_tinted_func_type copy_data32_tinted_func[2][4] = 
+	{	{
+			copy_data_tinted,
+			copy_data_tinted_8bitshift,
+			copy_data_tinted_16bitshift,
+			copy_data_tinted_24bitshift_masked /* to clear the sign bit ! */
+		},
+		{
+			copy_data_tinted_masked,
+			copy_data_tinted_8bitshift_masked,
+			copy_data_tinted_16bitshift_masked,
+			copy_data_tinted_24bitshift_masked
+		}
+	};
+
+	static copy_data32_func_type copy_data32_func[2][4] = 
+	{	{
+			copy_data32,
+			copy_data32_8bitshift,
+			copy_data32_16bitshift,
+			copy_data32_24bitshift_masked /* to clear the sign bit ! */
+		},
+		{
+			copy_data32_masked,
+			copy_data32_8bitshift_masked,
+			copy_data32_16bitshift_masked,
+			copy_data32_24bitshift_masked
+		}
+	};
+
 	if( size < ASStorageSlot_SIZE ) 
 		clear_flags( *flags, ASStorage_RLEDiffCompress );
 
@@ -533,7 +774,10 @@ compress_stored_data( ASStorage *storage, CARD8 *data, int size, ASFlagType *fla
 		{	
 			storage->comp_buf_size = ((size/AS_STORAGE_PAGE_SIZE)+1)*AS_STORAGE_PAGE_SIZE ;
 			storage->comp_buf = realloc( storage->comp_buf, storage->comp_buf_size );
-			storage->diff_buf = realloc( storage->diff_buf, storage->comp_buf_size*sizeof(short) );
+			storage->diff_buf = realloc( storage->diff_buf, storage->comp_buf_size*sizeof(ASStorageDiff) );
+#ifdef DEBUG_ALLOCS
+			show_debug( __FILE__,__FUNCTION__,__LINE__," realloced compression buffer to %d+%d*%d",storage->comp_buf_size, storage->comp_buf_size, sizeof(ASStorageDiff) );
+#endif 
 		}
 		buffer = storage->comp_buf ;
 		buf_size = storage->comp_buf_size ;
@@ -542,30 +786,28 @@ compress_stored_data( ASStorage *storage, CARD8 *data, int size, ASFlagType *fla
 			if( get_flags( *flags, ASStorage_Bitmap ) )
 			{	
 				if( get_flags( *flags, ASStorage_32Bit ) ) 
-				{	
+				{
 					uncompressed_size = size / 4 ;
-					if( get_flags( *flags, ASStorage_8BitShift ) ) 						   
-						bitmap_threshold = bitmap_threshold<<8 ;
+					if( get_flags( *flags, ASStorage_BitShift ) )
+						bitmap_threshold = bitmap_threshold<<ASStorage_Flags2Shift(*flags) ;
 					comp_size = rlediff_compress_bitmap32( buffer, data, uncompressed_size, bitmap_threshold );
 				}else
 					comp_size = rlediff_compress_bitmap8( buffer, data, uncompressed_size, bitmap_threshold );
 			}else 
 			{
-				short tint = bitmap_threshold ;
+				ASStorageDiff tint = bitmap_threshold ;
 				if( get_flags( *flags, ASStorage_32Bit ) ) 
 				{	
 					uncompressed_size = size / 4 ;
-					if( get_flags( *flags, ASStorage_8BitShift ) ) 						
-						compute_diff32_8bitshift( storage->diff_buf, data, uncompressed_size ); 	  
-					else
-						compute_diff32( storage->diff_buf, data, uncompressed_size ); 	  
+					compute_diff_func[get_flags(*flags,ASStorage_Masked)?1:0]
+					                 [ASStorage_Flags2ShiftIdx(*flags)](storage->diff_buf, data, uncompressed_size );
 				}else
 					compute_diff8( storage->diff_buf, data, uncompressed_size ); 	  
 				
 				if( tint < 255 )
 				{
 					int i;
-					short *diff = storage->diff_buf ; 
+					ASStorageDiff *diff = storage->diff_buf ; 
 					for( i = 0 ; i < uncompressed_size ; ++i ) 
 						diff[i] = (diff[i]*tint)/256 ;
 				}	 
@@ -599,25 +841,22 @@ compress_stored_data( ASStorage *storage, CARD8 *data, int size, ASFlagType *fla
 			{	
 				storage->comp_buf_size = ((size/AS_STORAGE_PAGE_SIZE)+1)*AS_STORAGE_PAGE_SIZE ;
 				storage->comp_buf = realloc( storage->comp_buf, storage->comp_buf_size );
-				storage->diff_buf = realloc( storage->diff_buf, storage->comp_buf_size*sizeof(short) );
+				storage->diff_buf = realloc( storage->diff_buf, storage->comp_buf_size*sizeof(ASStorageDiff) );
+#ifdef DEBUG_ALLOCS
+			show_debug( __FILE__,__FUNCTION__,__LINE__," realloced compression buffer to %d+%d*%d",storage->comp_buf_size, storage->comp_buf_size, sizeof(ASStorageDiff) );
+#endif 
+
 			}
 			buffer = storage->comp_buf ;
 			if( tint != 0x000000FF ) 
 			{	
-				if( get_flags( *flags, ASStorage_8BitShift ) )
-					for( comp_size = 0 ; comp_size < size ; ++comp_size )
-						buffer[comp_size] = (data32[comp_size]*tint)>>16 ;
-				else
-					for( comp_size = 0 ; comp_size < size ; ++comp_size )
-						buffer[comp_size] = (data32[comp_size]*tint)>>8 ;
+				copy_data32_tinted_func [get_flags(*flags,ASStorage_Masked)?1:0]
+					                 	[ASStorage_Flags2ShiftIdx(*flags)](buffer, data32, size, tint);
 			}else
 			{
-				if( get_flags( *flags, ASStorage_8BitShift ) )
-					for( comp_size = 0 ; comp_size < size ; ++comp_size )
-						buffer[comp_size] = (data32[comp_size]>>8) ;
-				else
-					for( comp_size = 0 ; comp_size < size ; ++comp_size )
-						buffer[comp_size] = data32[comp_size] ;
+				copy_data32_func [get_flags(*flags,ASStorage_Masked)?1:0]
+					           	 [ASStorage_Flags2ShiftIdx(*flags)](buffer, data32, size);
+
 			}	 
 		}else if( tint != 0x000000FF ) 
 		{
@@ -625,7 +864,10 @@ compress_stored_data( ASStorage *storage, CARD8 *data, int size, ASFlagType *fla
 			{	
 				storage->comp_buf_size = ((size/AS_STORAGE_PAGE_SIZE)+1)*AS_STORAGE_PAGE_SIZE ;
 				storage->comp_buf = realloc( storage->comp_buf, storage->comp_buf_size );
-				storage->diff_buf = realloc( storage->diff_buf, storage->comp_buf_size*sizeof(short) );
+				storage->diff_buf = realloc( storage->diff_buf, storage->comp_buf_size*sizeof(ASStorageDiff) );
+#ifdef DEBUG_ALLOCS
+			show_debug( __FILE__,__FUNCTION__,__LINE__," realloced compression buffer to %d+%d*%d",storage->comp_buf_size, storage->comp_buf_size, sizeof(ASStorageDiff) );
+#endif 
 			}
 			buffer = storage->comp_buf ;
 			for( comp_size = 0 ; comp_size < size ; ++comp_size )
@@ -661,9 +903,16 @@ static void
 add_storage_slots( ASStorageBlock *block )
 {
 	int i = block->slots_count ;
-	int size ; 
+	int size ;
+	int count = AS_STORAGE_SLOTS_BATCH ;  
 	LOCAL_DEBUG_OUT( "block = %p, block->slots = %p", block, block->slots );
-	block->slots_count += AS_STORAGE_SLOTS_BATCH ; 
+	if( block->slots_count + count  >= AS_STORAGE_MAX_SLOTS_CNT ) 
+	{
+		count = (int)AS_STORAGE_MAX_SLOTS_CNT - block->slots_count ; 
+		if( count < 0 ) 
+			return;
+	}
+	block->slots_count += count ; 
 	size = block->slots_count*sizeof(ASStorageSlot*)  ;
 	LOCAL_DEBUG_OUT( "block->slots_count = %d", block->slots_count );
 #ifndef DEBUG_ALLOCS
@@ -671,11 +920,14 @@ add_storage_slots( ASStorageBlock *block )
 	block->slots = realloc( block->slots, size);
 	LOCAL_DEBUG_OUT( "reallocated %d slots pointers", block->slots_count );
 #else
+	if( block->slots == NULL ) 
+		show_debug( __FILE__,__FUNCTION__,__LINE__,"allocating %d slots pointers", block->slots_count );
+	else
+		show_debug( __FILE__,__FUNCTION__,__LINE__,"reallocating %d slots pointers", block->slots_count );
 	block->slots = guarded_realloc( block->slots, block->slots_count*sizeof(ASStorageSlot*));
-	LOCAL_DEBUG_OUT( "reallocated %d slots pointers", block->slots_count );
 #endif
-	UsedMemory += AS_STORAGE_SLOTS_BATCH*sizeof(ASStorageSlot*) ;
-	memset( &(block->slots[i]),	0x00, AS_STORAGE_SLOTS_BATCH*sizeof(ASStorageSlot*) );
+	UsedMemory += count*sizeof(ASStorageSlot*) ;
+	memset( &(block->slots[i]),	0x00, count*sizeof(ASStorageSlot*) );
 }
 
 
@@ -690,16 +942,19 @@ create_asstorage_block( int useable_size )
 	if( allocate_size%AS_STORAGE_PAGE_SIZE > 0 ) 
 		allocate_size = ((allocate_size/AS_STORAGE_PAGE_SIZE)+1)*AS_STORAGE_PAGE_SIZE ;
 #ifndef DEBUG_ALLOCS
-	ptr = malloc(allocate_size);
+	ptr = calloc(1,allocate_size);
 #else
-	ptr = guarded_malloc(allocate_size);
-	LOCAL_DEBUG_OUT( "allocated %d files", allocate_size );
+	{
+		char msg[256];
+		ptr = guarded_calloc(1,allocate_size);
+		sprintf( msg, "allocated %d bytes, block = %p, total used = %d", allocate_size, ptr, UsedMemory + allocate_size );
+		PRINT_MEM_STATS(msg);
+	}
 #endif
 	UsedMemory += allocate_size ;
 	if( ptr == NULL ) 
 		return NULL;
 	block = ptr ;
-	memset( block, 0x00, sizeof(ASStorageBlock));
 	block->size = allocate_size - sizeof(ASStorageBlock) ;
 	block->total_free = block->size - ASStorageSlot_SIZE ;
 
@@ -710,6 +965,9 @@ create_asstorage_block( int useable_size )
 	{	
 		free( ptr ); 
 		UsedMemory -= allocate_size ;
+#ifdef DEBUG_ALLOCS
+		show_debug( __FILE__,__FUNCTION__,__LINE__,"freeing block %p, size = %d, total used = %d", ptr, allocate_size, UsedMemory );
+#endif
 		return NULL;
 	}
 	block->start = (ASStorageSlot*)((unsigned char*)ptr+((sizeof(ASStorageBlock)/ASStorageSlot_SIZE)+1)*ASStorageSlot_SIZE);
@@ -731,8 +989,6 @@ create_asstorage_block( int useable_size )
 static void
 destroy_asstorage_block( ASStorageBlock *block )
 {
-	LOCAL_DEBUG_OUT( "freeing block %p, size = %d", block, block->size );
-	
 	UsedMemory -= block->slots_count * sizeof(ASStorageSlot*) ;
 	UsedMemory -= block->size + sizeof(ASStorageBlock) ;
 
@@ -740,8 +996,13 @@ destroy_asstorage_block( ASStorageBlock *block )
 	free( block->slots );
 	free( block );	  
 #else	
-	guarded_free( block->slots );
-	guarded_free( block );
+	{
+		char msg[256];
+		sprintf( msg, "freeing block %p, size = %d, total used = %d", block, block->size, UsedMemory );
+		guarded_free( block->slots );
+		guarded_free( block );
+		PRINT_MEM_STATS(msg);
+	}
 #endif
 
 }
@@ -762,7 +1023,7 @@ select_storage_block( ASStorage *storage, int compressed_size, ASFlagType flags,
 		{	
 			if( block->total_free > compressed_size && 
 				block->total_free > AS_STORAGE_NOUSE_THRESHOLD && 
-				block->last_used < AS_STORAGE_MAX_SLOTS_CNT )
+				block->last_used+2 < AS_STORAGE_MAX_SLOTS_CNT  )
 				return i+1;
 		}else if( new_block < 0 ) 
 			new_block = i ;
@@ -776,7 +1037,7 @@ select_storage_block( ASStorage *storage, int compressed_size, ASFlagType flags,
 		storage->blocks = realloc( storage->blocks, storage->blocks_count*sizeof(ASStorageBlock*));
 #else
 		storage->blocks = guarded_realloc( storage->blocks, storage->blocks_count*sizeof(ASStorageBlock*));
-		LOCAL_DEBUG_OUT( "reallocated %d blocks pointers", storage->blocks_count );
+		show_debug( __FILE__,__FUNCTION__,__LINE__,"reallocated %d blocks pointers", storage->blocks_count );
 #endif		   
 		UsedMemory += 16*sizeof(ASStorageBlock*) ;
 
@@ -955,7 +1216,8 @@ select_storage_slot( ASStorageBlock *block, int size )
 			{
 				if( slot->flags == 0 )
 				{	  
-					int size_to_match = size+ASStorageSlot_SIZE ;
+					int single_slot_size = size+ASStorageSlot_SIZE ; 
+					int size_to_match = single_slot_size+ASStorageSlot_SIZE ;
 					++empty_slots_checked ;
 					
 					do
@@ -965,7 +1227,7 @@ select_storage_slot( ASStorageBlock *block, int size )
 							break;
 
 						LOCAL_DEBUG_OUT( "start = %p, slot = %p, slot->size = %ld, end = %p, size = %d, size_to_match = %d", block->start, slot, slot->size, block->end, size, size_to_match );
-						if((int)ASStorageSlot_USABLE_SIZE(slot) >= size )
+						if((int)ASStorageSlot_USABLE_SIZE(slot) >= single_slot_size )
 						{	
 							if( empty_slots_checked > 50 ) ++(block->long_searches);
 							return slot;
@@ -1040,7 +1302,7 @@ split_storage_slot( ASStorageBlock *block, ASStorageSlot *slot, int to_size )
 		LOCAL_DEBUG_OUT( "i = %d", i );
 		if( i >= max_i ) 
 		{
-			if( block->slots_count + AS_STORAGE_SLOTS_BATCH > AS_STORAGE_MAX_SLOTS_CNT )
+			if( block->slots_count >= AS_STORAGE_MAX_SLOTS_CNT )
 				return False;
 			else
 			{
@@ -1098,8 +1360,8 @@ store_data_in_block( ASStorageBlock *block, CARD8 *data, int size, int compresse
 	LOCAL_DEBUG_OUT( "block = %p", block );
 	if( !split_storage_slot( block, slot, compressed_size ) ) 
 	{
-		show_error( "storage slot split failed. Usable size = %d, desired size = %d", ASStorageSlot_USABLE_SIZE(slot), compressed_size+ASStorageSlot_SIZE );
-		return 0;
+		show_error( "failed to split storage to store data in block. Usable size = %d, desired size = %d", ASStorageSlot_USABLE_SIZE(slot), compressed_size+ASStorageSlot_SIZE );
+		return 0 ;
 	}
 	LOCAL_DEBUG_OUT( "block = %p", block );
 	block->total_free -= ASStorageSlot_FULL_SIZE(slot);
@@ -1148,8 +1410,11 @@ store_compressed_data( ASStorage *storage, CARD8* data, int size, int compressed
 			if( slot_id > 0 )	
 				id = make_asstorage_id( block_id, slot_id );
 			else
-				if( storage->blocks[block_id-1]->total_free >= compressed_size ) 
+				if( storage->blocks[block_id-1]->total_free >= compressed_size+ASStorageSlot_SIZE  ) 
+				{
+					show_error( "failed to store data in block. Total free size = %d, desired size = %d", storage->blocks[block_id-1]->total_free, compressed_size+ASStorageSlot_SIZE );
 					break;
+				}
 		}
 	}while( block_id != 0 && id == 0 );
 	return id ;		
@@ -1391,7 +1656,7 @@ card8_threshold( ASStorageDstBuffer *dst, void *src, size_t size)
 	dst->end = end ;
 }	 
 
-static inline int  
+static int  
 fetch_data_int( ASStorage *storage, ASStorageID id, ASStorageDstBuffer *buffer, int offset, int buf_size, CARD8 bitmap_value, 
 		  		data_cpy_func_type cpy_func, int *original_size)
 {
@@ -1467,6 +1732,24 @@ create_asstorage()
 	return storage ;
 }
 
+int 
+set_asstorage_block_size( ASStorage *storage, int new_size )
+{
+	int old_size ;
+	
+	if( storage == NULL ) 
+		storage = get_default_asstorage();
+	
+	old_size = storage->default_block_size ; 
+#if 1
+	if( new_size > AS_STORAGE_DEF_BLOCK_SIZE ) 
+		storage->default_block_size = new_size; 
+	else
+		storage->default_block_size = AS_STORAGE_DEF_BLOCK_SIZE; 
+#endif
+	return old_size;
+}
+
 void 
 destroy_asstorage(ASStorage **pstorage)
 {
@@ -1503,8 +1786,6 @@ destroy_asstorage(ASStorage **pstorage)
 	}
 }
 
-#define get_default_asstorage()   (_as_default_storage?_as_default_storage:(_as_default_storage=create_asstorage()))
-
 void 
 flush_default_asstorage()
 {
