@@ -23,7 +23,7 @@
 #endif
 
 #define LOCAL_DEBUG
-
+/*#define DEBUG_SL2XIMAGE */
 #include <string.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -42,52 +42,22 @@
 #endif
 #include "asvisual.h"
 
-#ifdef XSHMIMAGE
+#if defined(XSHMIMAGE) && !defined(X_DISPLAY_MISSING) 
 # include <sys/ipc.h>
 # include <sys/shm.h>
 # include <X11/extensions/XShm.h>
+#else
+# undef XSHMIMAGE
 #endif
 
-#ifdef HAVE_GLX
+#if defined(HAVE_GLX) && !defined(X_DISPLAY_MISSING) 
 # include <GL/gl.h>
 # include <GL/glx.h>
+#else
+# undef HAVE_GLX
 #endif
 
 
-CARD32 color2pixel32bgr(ASVisual *asv, CARD32 encoded_color, unsigned long *pixel);
-CARD32 color2pixel32rgb(ASVisual *asv, CARD32 encoded_color, unsigned long *pixel);
-CARD32 color2pixel24bgr(ASVisual *asv, CARD32 encoded_color, unsigned long *pixel);
-CARD32 color2pixel24rgb(ASVisual *asv, CARD32 encoded_color, unsigned long *pixel);
-CARD32 color2pixel16bgr(ASVisual *asv, CARD32 encoded_color, unsigned long *pixel);
-CARD32 color2pixel16rgb(ASVisual *asv, CARD32 encoded_color, unsigned long *pixel);
-CARD32 color2pixel15bgr(ASVisual *asv, CARD32 encoded_color, unsigned long *pixel);
-CARD32 color2pixel15rgb(ASVisual *asv, CARD32 encoded_color, unsigned long *pixel);
-CARD32 color2pixel_pseudo3bpp( ASVisual *asv, CARD32 encoded_color, unsigned long *pixel );
-CARD32 color2pixel_pseudo6bpp( ASVisual *asv, CARD32 encoded_color, unsigned long *pixel );
-CARD32 color2pixel_pseudo12bpp( ASVisual *asv, CARD32 encoded_color, unsigned long *pixel );
-
-void pixel2color32rgb(ASVisual *asv, unsigned long pixel, CARD32 *red, CARD32 *green, CARD32 *blue);
-void pixel2color32bgr(ASVisual *asv, unsigned long pixel, CARD32 *red, CARD32 *green, CARD32 *blue);
-void pixel2color24rgb(ASVisual *asv, unsigned long pixel, CARD32 *red, CARD32 *green, CARD32 *blue);
-void pixel2color24bgr(ASVisual *asv, unsigned long pixel, CARD32 *red, CARD32 *green, CARD32 *blue);
-void pixel2color16rgb(ASVisual *asv, unsigned long pixel, CARD32 *red, CARD32 *green, CARD32 *blue);
-void pixel2color16bgr(ASVisual *asv, unsigned long pixel, CARD32 *red, CARD32 *green, CARD32 *blue);
-void pixel2color15rgb(ASVisual *asv, unsigned long pixel, CARD32 *red, CARD32 *green, CARD32 *blue);
-void pixel2color15bgr(ASVisual *asv, unsigned long pixel, CARD32 *red, CARD32 *green, CARD32 *blue);
-
-void ximage2scanline32( ASVisual *asv, XImage *xim, ASScanline *sl, int y,  register unsigned char *xim_data );
-void ximage2scanline16( ASVisual *asv, XImage *xim, ASScanline *sl, int y,  register unsigned char *xim_data );
-void ximage2scanline15( ASVisual *asv, XImage *xim, ASScanline *sl, int y,  register unsigned char *xim_data );
-void ximage2scanline_pseudo3bpp( ASVisual *asv, XImage *xim, ASScanline *sl, int y,  register unsigned char *xim_data );
-void ximage2scanline_pseudo6bpp( ASVisual *asv, XImage *xim, ASScanline *sl, int y,  register unsigned char *xim_data );
-void ximage2scanline_pseudo12bpp( ASVisual *asv, XImage *xim, ASScanline *sl, int y,  register unsigned char *xim_data );
-
-void scanline2ximage32( ASVisual *asv, XImage *xim, ASScanline *sl, int y,  register unsigned char *xim_data );
-void scanline2ximage16( ASVisual *asv, XImage *xim, ASScanline *sl, int y,  register unsigned char *xim_data );
-void scanline2ximage15( ASVisual *asv, XImage *xim, ASScanline *sl, int y,  register unsigned char *xim_data );
-void scanline2ximage_pseudo3bpp( ASVisual *asv, XImage *xim, ASScanline *sl, int y,  register unsigned char *xim_data );
-void scanline2ximage_pseudo6bpp( ASVisual *asv, XImage *xim, ASScanline *sl, int y,  register unsigned char *xim_data );
-void scanline2ximage_pseudo12bpp( ASVisual *asv, XImage *xim, ASScanline *sl, int y,  register unsigned char *xim_data );
 
 #ifndef X_DISPLAY_MISSING
 static int  get_shifts (unsigned long mask);
@@ -107,8 +77,8 @@ debug_AllocColor( const char *file, const char *func, int line, ASVisual *asv, C
 {
 	Status sret ;
 	sret = XAllocColor( asv->dpy, cmap, pxcol );
-	show_progress( " XAllocColor in %s:%s():%d has %s -> cmap = %lX, pixel = %d, color = 0x%4.4lX, 0x%4.4lX, 0x%4.4lX",
-				   file, func, line, (sret==0)?"failed":"succeeded", cmap, pxcol->pixel, pxcol->red, pxcol->green, pxcol->blue );
+	show_progress( " XAllocColor in %s:%s():%d has %s -> cmap = %lX, pixel = %lu(%8.8lX), color = 0x%4.4X, 0x%4.4X, 0x%4.4X",
+				   file, func, line, (sret==0)?"failed":"succeeded", (long)cmap, (unsigned long)(pxcol->pixel), (unsigned long)(pxcol->pixel), pxcol->red, pxcol->green, pxcol->blue );
 	return sret;
 }
 #define ASV_ALLOC_COLOR(asv,cmap,pxcol)  debug_AllocColor(__FILE__, __FUNCTION__, __LINE__, (asv),(cmap),(pxcol))
@@ -180,7 +150,7 @@ prepare_scanline( unsigned int width, unsigned int shift, ASScanline *reusable_m
 	/* we want to align data by 8 byte boundary (double)
 	 * to allow for code with less ifs and easier MMX/3Dnow utilization :*/
 	aligned_width = width + (width&0x00000001);
-	sl->buffer = ptr = safemalloc (((aligned_width*4)+4)*sizeof(CARD32));
+	sl->buffer = ptr = safemalloc (((aligned_width*4)+16)*sizeof(CARD32));
 
 	sl->xc1 = sl->red 	= (CARD32*)(((long)ptr>>3)*8);
 	sl->xc2 = sl->green = sl->red   + aligned_width;
@@ -407,12 +377,12 @@ query_screen_visual_id( ASVisual *asv, Display *dpy, int screen, Window root, in
 	if( get_output_threshold() >= OUTPUT_VERBOSE_THRESHOLD )
 	{
 		fprintf( stderr, "Selected visual 0x%lx: depth %d, class %d\n RGB masks: 0x%lX, 0x%lX, 0x%lX, Byte Ordering: %s\n",
-				 asv->visual_info.visualid,
+				 (unsigned long)asv->visual_info.visualid,
 				 asv->visual_info.depth,
 				 asv->visual_info.class,
-				 asv->visual_info.red_mask,
-				 asv->visual_info.green_mask,
-				 asv->visual_info.blue_mask,
+				 (unsigned long)asv->visual_info.red_mask,
+				 (unsigned long)asv->visual_info.green_mask,
+				 (unsigned long)asv->visual_info.blue_mask,
 				 (ImageByteOrder(asv->dpy)==MSBFirst)?"MSBFirst":"LSBFirst" );
 	}
 #else
@@ -633,27 +603,27 @@ setup_truecolor_visual( ASVisual *asv )
 		return False;
 
 #ifdef HAVE_GLX
-	if( glXQueryExtension (dpy, NULL, NULL))
+	if( glXQueryExtension (asv->dpy, NULL, NULL))
 	{
 		int val = False;
-		glXGetConfig(dpy, vi, GLX_USE_GL, &val);		
+		glXGetConfig(asv->dpy, vi, GLX_USE_GL, &val);		
 		if( val ) 
 		{
-			asv->glx_scratch_gc_indirect = glXCreateContext (dpy, &(asv->visual_info), NULL, False);
+			asv->glx_scratch_gc_indirect = glXCreateContext (asv->dpy, &(asv->visual_info), NULL, False);
 			if( asv->glx_scratch_gc_indirect ) 
 			{	
 				set_flags( asv->glx_support, ASGLX_Available );
-				if( glXGetConfig(dpy, vi, GLX_RGBA, &val) == 0 )
+				if( glXGetConfig(asv->dpy, vi, GLX_RGBA, &val) == 0 )
 					if( val ) set_flags( asv->glx_support, ASGLX_RGBA );
-				if( glXGetConfig(dpy, vi, GLX_DOUBLEBUFFER, &val) == 0 )
+				if( glXGetConfig(asv->dpy, vi, GLX_DOUBLEBUFFER, &val) == 0 )
 					if( val ) set_flags( asv->glx_support, ASGLX_DoubleBuffer );
-				if( glXGetConfig(dpy, vi, GLX_DOUBLEBUFFER, &val) == 0 )
+				if( glXGetConfig(asv->dpy, vi, GLX_DOUBLEBUFFER, &val) == 0 )
 					if( val ) set_flags( asv->glx_support, ASGLX_DoubleBuffer );
 				
-				if( (asv->glx_scratch_gc_direct = glXCreateContext (dpy, &(asv->visual_info), NULL, True)) != NULL ) 
-					if( !glXIsDirect( dpy, asv->glx_scratch_gc_direct ) )
+				if( (asv->glx_scratch_gc_direct = glXCreateContext (asv->dpy, &(asv->visual_info), NULL, True)) != NULL ) 
+					if( !glXIsDirect( asv->dpy, asv->glx_scratch_gc_direct ) )
 					{	
-						glXDestroyContext(dpy, asv->glx_scratch_gc_direct );
+						glXDestroyContext(asv->dpy, asv->glx_scratch_gc_direct );
 						asv->glx_scratch_gc_direct = NULL ;
 					}
 #if 0                          /* that needs some more research :  */
@@ -1049,7 +1019,7 @@ LOCAL_DEBUG_OUT( "Colormap %lX, parent %lX, %ux%u%+d%+d, bw = %d, class %d",
 		** depths (such as on a Solaris box with 8bpp root window and
 		** 24bpp child windows), ParentRelative will not work. */
 		if ( get_flags(mask, CWBackPixmap) && attributes->background_pixmap == ParentRelative &&
-			 asv->visual_info.visual != DefaultVisual( dpy, DefaultScreen(dpy) ))
+			 asv->visual_info.visual != DefaultVisual( asv->dpy, DefaultScreen(asv->dpy) ))
 		{
 			clear_flags(mask, CWBackPixmap);
 		}
@@ -1093,7 +1063,9 @@ create_visual_pixmap( ASVisual *asv, Window root, unsigned int width, unsigned i
 	{	
 		if( root == None ) 
 			root = RootWindow(asv->dpy,DefaultScreen(asv->dpy));
-		p = XCreatePixmap( asv->dpy, root, MAX(width,(unsigned)1), MAX(height,(unsigned)1), (depth==0)?asv->true_depth:depth );
+		if( depth==0 )
+			depth = asv->true_depth ;
+		p = XCreatePixmap( asv->dpy, root, MAX(width,(unsigned)1), MAX(height,(unsigned)1), depth );
 	}
 	return p;
 #else
@@ -1114,6 +1086,85 @@ destroy_visual_pixmap( ASVisual *asv, Pixmap *ppmap )
 		}
 }
 
+#ifndef X_DISPLAY_MISSING
+static int
+quiet_xerror_handler (Display * dpy, XErrorEvent * error)
+{
+    return 0;
+}
+
+#endif
+
+int
+get_dpy_drawable_size (Display *drawable_dpy, Drawable d, unsigned int *ret_w, unsigned int *ret_h)
+{
+	int result = 0 ;
+#ifndef X_DISPLAY_MISSING
+	if( d != None && drawable_dpy != NULL ) 
+	{
+		Window        root;
+		unsigned int  ujunk;
+		int           junk;
+		int           (*oldXErrorHandler) (Display *, XErrorEvent *) = XSetErrorHandler (quiet_xerror_handler);
+		result = XGetGeometry (drawable_dpy, d, &root, &junk, &junk, ret_w, ret_h, &ujunk, &ujunk);
+		XSetErrorHandler (oldXErrorHandler);
+	}
+#endif
+	if ( result == 0)
+	{
+		*ret_w = 0;
+		*ret_h = 0;
+		return 0;
+	}
+	return 1;
+}
+
+Bool
+get_dpy_window_position (Display *window_dpy, Window root, Window w, int *px, int *py, int *transparency_x, int *transparency_y)
+{
+	Bool result = False ;
+	int x = 0, y = 0, transp_x = 0, transp_y = 0 ;
+#ifndef X_DISPLAY_MISSING
+	if( window_dpy != NULL && w != None ) 
+	{
+		Window wdumm;
+		int rootHeight = XDisplayHeight(window_dpy, DefaultScreen(window_dpy) );
+		int rootWidth = XDisplayWidth(window_dpy, DefaultScreen(window_dpy) );
+
+		if( root == None ) 
+			root = RootWindow(window_dpy,DefaultScreen(window_dpy));
+			
+		result = XTranslateCoordinates (window_dpy, w, root, 0, 0, &x, &y, &wdumm);
+		if( result ) 
+		{
+			/* taking in to consideration virtual desktopping */
+			result = (x < rootWidth && y < rootHeight );
+			if( result )
+			{
+				unsigned int width = 0, height = 0;
+				get_dpy_drawable_size (window_dpy, w, &width, &height);				
+				result = (x + width > 0 && y+height > 0) ; 
+			}
+
+			for( transp_x = x ; transp_x < 0 ; transp_x += rootWidth ); 			
+			for( transp_y = y ; transp_y < 0 ; transp_y += rootHeight ); 			
+			while( transp_x > rootWidth ) transp_x -= rootWidth ; 
+			while( transp_y > rootHeight ) transp_y -= rootHeight ; 
+		}
+	}
+#endif
+	if( px ) 
+		*px = x;
+	if( py ) 
+		*py = y;
+	if( transparency_x ) 
+		*transparency_x = transp_x ; 
+	if( transparency_y ) 
+		*transparency_y = transp_y ; 
+	return result;
+}
+
+
 #ifndef X_DISPLAY_MISSING
 static unsigned char *scratch_ximage_data = NULL ;
 static int scratch_use_count = 0 ;
@@ -1180,6 +1231,7 @@ typedef struct ASXShmImage
 	int 			 ref_count ;
 	Bool			 wait_completion_event ;
 	unsigned int 	 size ;
+  ASVisual *asv ;
 }ASXShmImage;
 
 typedef struct ASShmArea
@@ -1296,7 +1348,7 @@ destroy_xshmimage_segment(ASHashableValue value, void *data)
 	if( img_data->segment != NULL )
 	{
 		LOCAL_DEBUG_OUT( "XSHMIMAGE> FREE_SEG : img_data = %p : segent to be freed: shminfo = %p ", img_data, img_data->segment );
-		XShmDetach (dpy, img_data->segment);
+		XShmDetach (img_data->asv->dpy, img_data->segment);
 		save_shm_area( img_data->segment->shmaddr, img_data->segment->shmid, img_data->size );
 		free( img_data->segment );
 		img_data->segment = NULL ;
@@ -1408,10 +1460,11 @@ ximage2shmseg( XImage *xim )
 	return 0; 
 }	 
 
-void registerXShmImage( XImage *ximage, XShmSegmentInfo* shminfo )
+void registerXShmImage( ASVisual *asv, XImage *ximage, XShmSegmentInfo* shminfo )
 {
 	ASXShmImage *data = safecalloc( 1, sizeof(ASXShmImage));
 	LOCAL_DEBUG_OUT( "XSHMIMAGE> CREATE_XIM : img_data = %p : image created: xiom = %p, shminfo = %p, segment = %d, data = %p", data, ximage, shminfo, shminfo->shmid, ximage->data );
+  data->asv = asv ;
 	data->ximage = ximage ;
 	data->segment = shminfo ;
 	data->size = ximage->bytes_per_line * ximage->height ;
@@ -1448,14 +1501,14 @@ Bool ASPutXImage( ASVisual *asv, Drawable d, GC gc, XImage *xim,
 
 	if( ( img_data = check_XImage_shared( xim )) != NULL )
 	{
-		LOCAL_DEBUG_OUT( "XSHMIMAGE> PUT_XIM : using shared memory Put = %p", xim );
+/*		LOCAL_DEBUG_OUT( "XSHMIMAGE> PUT_XIM : using shared memory Put = %p", xim ); */
 		if( XShmPutImage( asv->dpy, d, gc, xim, src_x, src_y, dest_x, dest_y,width, height, True ) )
 		{
 			img_data->wait_completion_event = True ;
 			return True ;
 		}
 	}
-	LOCAL_DEBUG_OUT( "XSHMIMAGE> PUT_XIM : using normal Put = %p", xim );
+/*	LOCAL_DEBUG_OUT( "XSHMIMAGE> PUT_XIM : using normal Put = %p", xim ); */
 	return XPutImage( asv->dpy, d, gc, xim, src_x, src_y, dest_x, dest_y,width, height );
 }
 
@@ -1473,7 +1526,7 @@ XImage *ASGetXImage( ASVisual *asv, Drawable d,
 		Window        root;
 		unsigned int  ujunk;
 		int           junk;
-		if(XGetGeometry (dpy, d, &root, &junk, &junk, &ujunk, &ujunk, &ujunk, &depth) == 0)
+		if(XGetGeometry (asv->dpy, d, &root, &junk, &junk, &ujunk, &ujunk, &ujunk, &depth) == 0)
 			return NULL ;
 
 		xim = create_visual_ximage(asv,width,height,depth);
@@ -1547,7 +1600,10 @@ create_visual_ximage( ASVisual *asv, unsigned int width, unsigned int height, un
 #if 0
 	unit = asv->dpy->bitmap_unit;
 #else
-	unit = (asv->true_depth+7)&0x0038;
+	if( depth == 0 ) 
+		unit = (asv->true_depth+7)&0x0038;
+	else
+		unit = (depth+7)&0x0038;
 	if( unit == 24 )
 		unit = 32 ;
 #endif
@@ -1582,7 +1638,7 @@ create_visual_ximage( ASVisual *asv, unsigned int width, unsigned int height, un
 			{
 				shminfo->readOnly = False;
 				XShmAttach (asv->dpy, shminfo);
-				registerXShmImage( ximage, shminfo );
+				registerXShmImage( asv, ximage, shminfo );
 			}
 		}
 	}
@@ -1626,7 +1682,10 @@ create_visual_scratch_ximage( ASVisual *asv, unsigned int width, unsigned int he
 #if 0
 	unit = asv->dpy->bitmap_unit;
 #else
-	unit = (asv->true_depth+7)&0x0038;
+	if( depth == 0 )
+		unit = (asv->true_depth+7)&0x0038;
+	else
+		unit = (depth+7)&0x0038;
 	if( unit == 24 )
 		unit = 32 ;
 #endif
@@ -1807,37 +1866,32 @@ void pixel2color15rgb(ASVisual *asv, unsigned long pixel, CARD32 *red, CARD32 *g
 void pixel2color15bgr(ASVisual *asv, unsigned long pixel, CARD32 *red, CARD32 *green, CARD32 *blue)
 {}
 
-#ifndef X_DISPLAY_MISSING
 void ximage2scanline32(ASVisual *asv, XImage *xim, ASScanline *sl, int y,  register unsigned char *xim_data )
 {
 	register CARD32 *r = sl->xc1+sl->offset_x, *g = sl->xc2+sl->offset_x, *b = sl->xc3+sl->offset_x;
 	register CARD32 *a = sl->alpha+sl->offset_x;
-	register int i = MIN((unsigned int)(xim->width),sl->width-sl->offset_x);
-	register CARD8 *src = (CARD8*)(xim_data+(i-1)*4) ;
+	int max_i = MIN((unsigned int)(xim->width),sl->width-sl->offset_x);
+	register CARD32 *src = (CARD32*)xim_data ;
+	register int i = 0;
 	if( asv->msb_first )
+	{
 		do
 		{
-			--i ;
-			{
-				a[i] = src[0];
-				r[i] = src[1];
-				g[i] = src[2];
-				b[i] = src[3];
-				src -= 4 ;
-			}
-		}while(i);
-	else
+			r[i] = (src[i]>>8)&0x0ff;
+			g[i] = (src[i]>>16)&0x0ff;
+			b[i] = (src[i]>>24)&0x0ff;
+			a[i] = src[i]&0x0ff;
+		}while(++i < max_i);
+	}else
+	{
 		do
 		{
-			--i ;
-			{
-				a[i] = src[3];
-				r[i] = src[2];
-				g[i] = src[1];
-				b[i] = src[0];
-				src -= 4 ;
-			}
-		}while(i);
+			r[i] = (src[i]>>16)&0x0ff;
+			g[i] = (src[i]>>8)&0x0ff;
+			b[i] =  src[i]&0x0ff;
+			a[i] = (src[i]>>24)&0x0ff;
+		}while(++i < max_i);
+	}
 }
 
 void ximage2scanline16( ASVisual *asv, XImage *xim, ASScanline *sl, int y,  register unsigned char *xim_data )
@@ -1894,6 +1948,8 @@ void ximage2scanline15( ASVisual *asv, XImage *xim, ASScanline *sl, int y,  regi
 		}while( --i >= 0);
 }
 
+#ifndef X_DISPLAY_MISSING
+
 void
 ximage2scanline_pseudo3bpp( ASVisual *asv, XImage *xim, ASScanline *sl, int y,  register unsigned char *xim_data )
 {
@@ -1995,7 +2051,7 @@ ximage2scanline_pseudo12bpp( ASVisual *asv, XImage *xim, ASScanline *sl, int y,
 			}
 		}while( --i >= 0);
 }
-
+#endif
 
 void scanline2ximage32( ASVisual *asv, XImage *xim, ASScanline *sl, int y,  register unsigned char *xim_data )
 {
@@ -2157,6 +2213,7 @@ void scanline2ximage15( ASVisual *asv, XImage *xim, ASScanline *sl, int y,  regi
 	}
 }
 
+#ifndef X_DISPLAY_MISSING
 void
 scanline2ximage_pseudo3bpp( ASVisual *asv, XImage *xim, ASScanline *sl, int y,  register unsigned char *xim_data )
 {
@@ -2295,6 +2352,6 @@ scanline2ximage_pseudo12bpp( ASVisual *asv, XImage *xim, ASScanline *sl, int y,
 		}while(i);
 	}
 }
+
 #endif /* ifndef X_DISPLAY_MISSING */
 
-
