@@ -651,7 +651,8 @@ clip_line( int k, int x0, int y0, int cw, int ch, int *x, int*y )
 	if( new_x < 0 ) 
 	{
 		new_x = 0 ;
-		new_y = (-x0 / k) + y0 ;
+		if( k != 0 ) 
+			new_y = (-x0 / k) + y0 ;
 	}	 
 	if( new_y < 0 ) 
 	{
@@ -665,9 +666,12 @@ clip_line( int k, int x0, int y0, int cw, int ch, int *x, int*y )
 	if( new_x >= cw ) 
 	{
 		new_x = cw - 1 ;
-		new_y = (new_x - x0)/k  + y0 ; 
-		if( new_y < 0 ) 
-			return False;	   
+		if( k != 0 )
+		{
+			new_y = (new_x - x0)/k  + y0 ; 
+			if( new_y < 0 ) 
+				return False;	   
+		}
 	}		 
 	if( new_y >= ch ) 
 	{
