@@ -54,6 +54,7 @@ typedef enum
 	ASIT_Gif,
 	ASIT_Tiff,
 	ASIT_XMLScript,
+	ASIT_SVG,
 	/* reserved for future implementation : */
 	ASIT_Xbm,
 	ASIT_Targa,
@@ -112,6 +113,9 @@ typedef struct ASImageListEntry
 #define AS_IMPORT_RESIZED		(AS_IMPORT_RESIZE_V|AS_IMPORT_RESIZE_H)
 #define AS_IMPORT_SCALED_H		(0x01<<3)      /* if unset - then tile */
 #define AS_IMPORT_SCALED_V		(0x01<<4)      /* if unset - then tile */
+#define AS_IMPORT_SCALED_BOTH	(AS_IMPORT_SCALED_H|AS_IMPORT_SCALED_V)
+#define AS_IMPORT_FAST			(0x01<<5)      /* can sacrifice quality for speed */
+
 
 typedef struct ASImageImportParams
 {
@@ -124,6 +128,9 @@ typedef struct ASImageImportParams
 	ASAltImFormats 	format ;
 	unsigned int 	compression ;
 	char 		  **search_path ;                 /* NULL terminated list  */
+	
+	int 			return_animation_delay ;
+	int 			return_animation_repeats ;
 }ASImageImportParams;
 
 typedef ASImage* (*as_image_loader_func)( const char * path, ASImageImportParams *params );
@@ -141,6 +148,7 @@ ASImage *gif2ASImage ( const char * path, ASImageImportParams *params );
 ASImage *tiff2ASImage( const char * path, ASImageImportParams *params );
 ASImage *xml2ASImage ( const char * path, ASImageImportParams *params );
 ASImage *tga2ASImage ( const char * path, ASImageImportParams *params );
+ASImage *svg2ASImage ( const char * path, ASImageImportParams *params );
 ASImage *convert_argb2ASImage( ASVisual *asv, int width, int height, ARGB32 *argb, CARD8 *gamma_table );
 ASImage *argb2ASImage( const char *path, ASImageImportParams *params );
 
@@ -208,6 +216,12 @@ ASImage *file2ASImage( const char *file, ASFlagType what, double gamma, unsigned
 ASImage *file2ASImage_extra( const char *file, ASImageImportParams *params );
 ASImage *get_asimage( ASImageManager* imageman, const char *file, ASFlagType what, unsigned int compression );
 ASImage *get_asimage_extra( ASImageManager* imageman, const char *file, ASImageImportParams *params );
+ASImageFileTypes get_asimage_file_type( ASImageManager* imageman, const char *file );
+
+#define AS_THUMBNAIL_PROPORTIONAL 		(0x01<<0)
+#define AS_THUMBNAIL_DONT_ENLARGE 		(0x01<<1)
+#define AS_THUMBNAIL_DONT_REDUCE		(0x01<<2)
+ASImage *get_thumbnail_asimage( ASImageManager* imageman, const char *file, int thumb_width, int thumb_height, ASFlagType flags /* see above */);
 
 ASImageFileTypes check_asimage_file_type( const char *realfilename );
 
