@@ -56,6 +56,7 @@ struct SyntaxDef;
 struct TermDef;
 struct ASSession;
 struct ASDesktopCategory;
+struct ASDesktopEntry;
 struct ASCategoryTree;
 struct ASDatabase;
 
@@ -130,6 +131,21 @@ void  handler_set_geometry( char *argv, void *trg, long param );
 void  handler_set_gravity( char *argv, void *trg, long param );
 
 
+#define ASXMLVAR_IconButtonWidth 		"icon.button.width"
+#define ASXMLVAR_IconButtonHeight		"icon.button.height"
+#define ASXMLVAR_IconWidth				"icon.width"
+#define ASXMLVAR_IconHeight				"icon.height"
+#define ASXMLVAR_MinipixmapWidth		"minipixmap.width"
+#define ASXMLVAR_MinipixmapHeight		"minipixmap.height"
+#define ASXMLVAR_TitleFontSize			"title.font.size"
+#define ASXMLVAR_MenuFontSize			"menu.font.size"
+#define ASXMLVAR_MenuShowMinipixmaps 	"menu.show_minipixmaps"
+#define ASXMLVAR_MenuShowUnavailable 	"menu.show_unavailable"
+#define ASXMLVAR_MenuTxtItemsInd		"menu.texture_items_individualy"
+#define ASXMLVAR_MenuRecentSubmenuItems "menu.recent_submenu_items"
+#define ASXMLVAR_MenuFolderPixmap		"menu.folder_pixmap"
+
+
 typedef struct ASProgArgs
 {
 	/* backup of the cmdline : */
@@ -196,6 +212,10 @@ typedef struct ASEnvironment
   unsigned short desk_pages_h, desk_pages_v ;
   unsigned short desk_scale ;
 
+	enum{ ASE_AllowModuleNameCollision = 0,
+		  ASE_KillOldModuleOnNameCollision,	
+		  ASE_KillNewModuleOnNameCollision 
+		}module_name_collision ; 
   char *tool_command[ASTool_Count] ;
   
   char *gtkrc_path ;
@@ -222,6 +242,9 @@ void destroy_asenvironment( ASEnvironment **penv );
 
 #define MAX_SINGLETONS_NUM 				32
 #define BACKGROUND_SINGLETON_ID 		(MAX_SINGLETONS_NUM-1)
+/* :sG:Audio: 
+  This Audio entry may need to be removed/repalced.
+    Since its related to the Audio module. and Sound2 will be replacing it. */
 #define AUDIO_SINGLETON_ID 				(MAX_SINGLETONS_NUM-2)
 #define TAR_SINGLETON_ID 				(MAX_SINGLETONS_NUM-3)
 #define MAX_USER_SINGLETONS_NUM 		(MAX_SINGLETONS_NUM-3)
@@ -250,8 +273,9 @@ void SetMyName (char *argv0);
 void override_environ( char **envp );
 void destroy_asdatabase();
 
-struct ASDesktopCategory *name2desktop_category( const char *name, 
-									struct ASCategoryTree **tree_return );
+struct ASDesktopCategory* name2desktop_category( const char *name, struct ASCategoryTree **tree_return );
+struct ASDesktopEntry* name2desktop_entry( const char *name, struct ASCategoryTree **tree_return );
+									
 
 void free_as_app_args();
 void FreeMyAppResources();
@@ -328,7 +352,7 @@ extern struct ASCategoryTree *StandardCategories ;
 extern struct ASCategoryTree *AfterStepCategories ;
 extern struct ASCategoryTree *KDECategories ;
 extern struct ASCategoryTree *GNOMECategories ;
-extern struct ASCategoryTree *SystemCategories ;
+extern struct ASCategoryTree *OtherCategories ;
 extern struct ASCategoryTree *CombinedCategories ;
 
 /* this two are unused in as-stable yet : */
