@@ -576,6 +576,12 @@ create_astbar ()
     tbar->rendered_root_x = tbar->rendered_root_y = 0xFFFF;
     tbar->composition_method[0] = TEXTURE_TRANSPIXMAP_ALPHA ;
     tbar->composition_method[1] = TEXTURE_TRANSPIXMAP_ALPHA ;
+	/* since saturation is allowed to take values from 0 to 100 
+	   inclusive - we use -1 to indicate absence : */
+	tbar->sat[0] = -1 ; 
+	tbar->sat[1] = -1 ; 
+	tbar->hue[0] = -1 ; 
+	tbar->hue[1] = -1 ; 
 	return tbar;
 }
 
@@ -586,7 +592,22 @@ flush_tbar_backs(ASTBarData *tbar)
     for (i = 0; i < BAR_STATE_NUM; ++i)
         if (tbar->back[i])
             destroy_asimage (&(tbar->back[i]));
+    set_flags( tbar->state, BAR_FLAGS_REND_PENDING );
 }
+
+static inline void
+flush_tbar_state_backs(ASTBarData *tbar, int state)
+{
+	if( state < 0 || state > BAR_STATE_NUM ) 
+		flush_tbar_backs(tbar);
+	else
+	{
+    	if ( tbar->back[state] )
+			destroy_asimage (&(tbar->back[state]));
+	    set_flags( tbar->state, BAR_FLAGS_REND_PENDING );
+	}
+}
+
 void
 destroy_astbar (ASTBarData ** ptbar)
 {
@@ -608,11 +629,8 @@ destroy_astbar (ASTBarData ** ptbar)
                 free( tbar->tiles );
             }
 
-            for (i = 0; i < BAR_STATE_NUM; ++i)
-			{
-				if (tbar->back[i])
-					destroy_asimage (&(tbar->back[i]));
-            }
+			flush_tbar_backs(tbar);
+
             if( tbar == FocusedBar )
             {
                 withdraw_balloon( NULL );
@@ -723,14 +741,7 @@ set_astbar_size (ASTBarData * tbar, unsigned int width, unsigned int height)
 		tbar->width = w;
 		tbar->height = h;
 		if (changed)
-		{
-			register int  i;
-
-			for (i = 0; i < BAR_STATE_NUM; ++i)
-				if (tbar->back[i])
-					destroy_asimage (&(tbar->back[i]));
-            set_flags( tbar->state, BAR_FLAGS_REND_PENDING );
-		}
+			flush_tbar_backs(tbar);
 	}
 	return changed;
 }
@@ -814,14 +825,19 @@ set_astile_styles( ASTBarData *tbar, ASTile *tile, int state )
 }
 
 Bool
-set_astbar_style_ptr (ASTBarData * tbar, unsigned int state, MyStyle *style)
+set_astbar_style_ptr (ASTBarData * tbar, int state, MyStyle *style)
 {
 	Bool          changed = False;
 LOCAL_DEBUG_OUT( "bar(%p)->state(%d)->style_name(\"%s\")", tbar, state, style?style->name:"" );
     if (tbar && state < BAR_STATE_NUM )
 	{
-        changed = (style != tbar->style[state]);
-		tbar->style[state] = style;
+	    register int i ;
+    	for (i = 0; i < BAR_STATE_NUM; ++i)
+        	if(i == state || state == -1 )
+			{
+        		changed = changed || (style != tbar->style[i]);
+				tbar->style[i] = style;
+			}
 /*LOCAL_DEBUG_OUT( "style(%p)->changed(%d)->tiles_num(%d)", style, changed, tbar->tiles_num );*/
         if (changed )
 		{
@@ -829,12 +845,7 @@ LOCAL_DEBUG_OUT( "bar(%p)->state(%d)->style_name(\"%s\")", tbar, state, style?st
             while ( --i >= 0 )
                 set_astile_styles( tbar, &(tbar->tiles[i]), state );
 
-            if( tbar->back[state] )
-            {
-                destroy_asimage (&(tbar->back[state]));
-                tbar->back[state] = NULL;
-            }
-            set_flags( tbar->state, BAR_FLAGS_REND_PENDING );
+			flush_tbar_state_backs(tbar, state);
             update_astbar_bevel_size (tbar);
         }
     }
@@ -855,12 +866,8 @@ set_astbar_flip( ASTBarData * tbar, int flip )
 /*LOCAL_DEBUG_OUT( "style(%p)->changed(%d)->tiles_num(%d)", style, changed, tbar->tiles_num );*/
         if (changed )
 		{
-			int i ;
-			set_flags(tbar->state, vert_flag);
-			for (i = 0; i < BAR_STATE_NUM; ++i)
-	            if( tbar->back[i] )
-                	destroy_asimage (&(tbar->back[i]));
-            set_flags( tbar->state, BAR_FLAGS_REND_PENDING );
+			flush_tbar_backs(tbar);
+			set_flags( tbar->state, vert_flag );
         }
     }
 	return changed;
@@ -892,12 +899,7 @@ invalidate_astbar_style (ASTBarData * tbar, int state)
 		{
 			changed = True;
 			tbar->style[state] = NULL;
-			if( tbar->back[state] )
-			{
-				destroy_asimage (&(tbar->back[state]));
-				tbar->back[state] = NULL;
-			}
-			set_flags( tbar->state, BAR_FLAGS_REND_PENDING );
+			flush_tbar_state_backs(tbar, state);
 		}
 	}
     return changed;
@@ -1064,8 +1066,8 @@ add_astbar_icon( ASTBarData * tbar, unsigned char col, unsigned char row, int fl
 
         if( flip == 0 )
         {
-            if( (tile->data.image.im = dup_asimage( icon )) == NULL )
-                    tile->data.image.im = clone_asimage( icon, 0xFFFFFFFF );
+            if( icon->imageman == NULL || (tile->data.image.im = dup_asimage( icon )) == NULL )
+            	tile->data.image.im = clone_asimage( icon, 0xFFFFFFFF );
         }else
 		{
 			int dst_width = icon->width, dst_height = icon->height ;
@@ -1203,20 +1205,12 @@ move_astbar (ASTBarData * tbar, ASCanvas * pc, int win_x, int win_y)
 		tbar->root_y = root_y;
 		if( changed )
 		{
-			Bool redraw = False ;
 			register int  i = BAR_STATE_NUM;
 			while (--i >= 0)
 			{
 				if (tbar->style[i] && TransparentMS(tbar->style[i]))
-            	{
-					if (tbar->back[i])
-						destroy_asimage (&(tbar->back[i]));
-                	if( i == ((tbar->state)&BAR_STATE_FOCUS_MASK) )
-                    	redraw = True;
-            	}
+					flush_tbar_state_backs(tbar, i);
 			}
-        	if( redraw )
-            	set_flags( tbar->state, BAR_FLAGS_REND_PENDING );
 		}
 		changed = changed || (win_x != tbar->win_x || win_y != tbar->win_y);
 		tbar->win_x = win_x;
@@ -1307,7 +1301,6 @@ update_astbar_transparency (ASTBarData * tbar, ASCanvas * pc, Bool force)
 {
 	int           root_x, root_y;
     Bool          changed = False;
-    Bool          redraw = False;
 
 	if (tbar == NULL || pc == NULL)
         return False;;
@@ -1322,19 +1315,10 @@ update_astbar_transparency (ASTBarData * tbar, ASCanvas * pc, Bool force)
 		tbar->root_y = root_y;
 
 		while (--i >= 0)
-		{
 			if (tbar->style[i] && TransparentMS(tbar->style[i]))
-            {
-				if (tbar->back[i])
-					destroy_asimage (&(tbar->back[i]));
-                if( i == ((tbar->state)&BAR_STATE_FOCUS_MASK) )
-                    redraw = True;
-            }
-		}
-        if( redraw )
-            set_flags( tbar->state, BAR_FLAGS_REND_PENDING );
+				flush_tbar_state_backs(tbar, i);
     }
-    return redraw;
+    return get_flags( tbar->state, BAR_FLAGS_REND_PENDING );
 }
 
 Bool
@@ -1390,28 +1374,11 @@ trim_astbar_grid_dim( short *dim, int size, int space_left )
 }
 
 
-#ifdef TRACE_render_astbar
-#undef render_astbar
-Bool render_astbar (ASTBarData * tbar, ASCanvas * pc);
-Bool
-trace_render_astbar (ASTBarData * tbar, ASCanvas * pc, const char *file, int line)
-{
-    Bool res;
-    fprintf (stderr, "D>%s(%d):render_astbar(%p,%p)\n", file, line, tbar, pc);
-    res = render_astbar (tbar, pc);
-	fprintf (stderr,
-             "D>%s(%d):render_astbar(%p,%p) returned %d\n", file, line, tbar, pc, res);
-    if( tbar && res <= 0 )
-        fprintf( stderr, "D>%s(%d):render_astbar tbar data: state %lX, %ux%u%+d%+d, root %+d%+d, styles %p,%p, tiles_num %d, tiles %p, canvas %X\n" ,
-                 file, line, tbar->state, tbar->width, tbar->height, tbar->win_x, tbar->win_y, tbar->root_x, tbar->root_y, tbar->style[0], tbar->style[1], tbar->tiles_num, tbar->tiles, pc->canvas );
-    return res;
-}
-#endif
-Bool
-render_astbar (ASTBarData * tbar, ASCanvas * pc)
+static inline Bool
+render_astbar_int (ASTBarData * tbar, ASCanvas * pc, ASImage **pcache, ASCanvas *origin_canvas)
 {
     START_TIME(started);
-    ASImage      *back;
+    ASImage      *back = NULL;
 	MyStyle      *style;
 	ASImageBevel  bevel;
     ASImageLayer *layers;
@@ -1447,34 +1414,74 @@ LOCAL_DEBUG_OUT("style(%p)->geom(%ux%u%+d%+d)->hilite(0x%X)", style, tbar->width
 
     if (style == NULL)
         return -2;
-    
+
+	if( origin_canvas == NULL ) 
+		origin_canvas = pc ;
+		    
     mystyle_make_bevel (style, &bevel, tbar->hilite[state], get_flags (tbar->state, BAR_STATE_PRESSED_MASK));
     h_bevel_size = bevel.left_outline+bevel.right_outline ;
     v_bevel_size = bevel.top_outline+bevel.bottom_outline ;
 
     /* validating our images : */
-	if ((back = tbar->back[state]) != NULL)
+	if ( tbar->back[state] != NULL)
 	{
         if( tbar->root_x != pc->root_x + (int)pc->bw + tbar->win_x ||
             tbar->root_y != pc->root_y + (int)pc->bw + tbar->win_y )
+		{
             update_astbar_transparency( tbar, pc, False );
-
+		}
+	}
+	if( (back = tbar->back[state]) != NULL )
+	{
         if (back->width != tbar->width || back->height != tbar->height ||
 			((tbar->rendered_root_x != tbar->root_x || tbar->rendered_root_y != tbar->root_y) &&
 			 style->texture_type > TEXTURE_PIXMAP))
 		{
-			destroy_asimage (&(tbar->back[state]));
-			tbar->back[state] = back = NULL;
+			flush_tbar_state_backs(tbar, state);
+			back = NULL;
 		}
 	}
 LOCAL_DEBUG_OUT("back(%p), vertical?%s", back, get_flags( tbar->state, BAR_FLAGS_VERTICAL )?"Yes":"No" );
 	if (back == NULL)
 	{
-        back = mystyle_make_image (style, 
-                                   tbar->root_x+bevel.left_outline, 
-                                   tbar->root_y+bevel.top_outline , 
-                                   tbar->width, tbar->height, 
-                                   get_flags( tbar->state, BAR_FLAGS_VERTICAL )?FLIP_VERTICAL:0);
+		if( (get_flags( tbar->state, BAR_FLAGS_CROP_UNFOCUSED_BACK ) && !IsASTBarFocused(tbar)) ||
+			(get_flags( tbar->state, BAR_FLAGS_CROP_FOCUSED_BACK ) && IsASTBarFocused(tbar)))
+		{
+			if( pcache ) 
+			{
+				if( *pcache == NULL ) 
+				{
+					*pcache = mystyle_make_image (style, 
+                                	   origin_canvas->root_x, 
+                                	   origin_canvas->root_y, 
+                                	   origin_canvas->width+origin_canvas->bw, origin_canvas->height+origin_canvas->bw, 
+                                	   get_flags( tbar->state, BAR_FLAGS_VERTICAL )?FLIP_VERTICAL:0);
+				}
+				if( *pcache ) 
+					back = tile_asimage( ASDefaultVisual, *pcache,
+										   tbar->root_x - origin_canvas->root_x + bevel.top_outline, 
+										   tbar->root_y - origin_canvas->root_y + bevel.left_outline, 
+                                		   tbar->width, tbar->height, 
+											TINT_LEAVE_SAME,
+											ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT );
+			}
+			
+			if( back == NULL ) 
+        		back = mystyle_crop_image (style, 
+                                		   pc->root_x, 
+                                		   pc->root_y, 
+										   tbar->root_x - origin_canvas->root_x + bevel.top_outline, 
+										   tbar->root_y - origin_canvas->root_y + bevel.left_outline, 
+                                		   tbar->width, tbar->height, 
+										   origin_canvas->width+(int)origin_canvas->bw, 
+										   origin_canvas->height+(int)origin_canvas->bw, 
+                                		   get_flags( tbar->state, BAR_FLAGS_VERTICAL )?FLIP_VERTICAL:0);
+		}else
+        	back = mystyle_make_image (style, 
+                                	   tbar->root_x+bevel.left_outline, 
+                                	   tbar->root_y+bevel.top_outline , 
+                                	   tbar->width, tbar->height, 
+                                	   get_flags( tbar->state, BAR_FLAGS_VERTICAL )?FLIP_VERTICAL:0);
 		tbar->back[state] = back ;
 LOCAL_DEBUG_OUT("back-try2(%p)", back );
 		if (back == NULL)
@@ -1688,6 +1695,8 @@ LOCAL_DEBUG_OUT("back-try2(%p)", back );
 #endif
 	if( get_flags( ASDefaultVisual->glx_support, ASGLX_UseForImageTx ) )
 		fmt = ASA_ASImage ;
+
+    LOCAL_DEBUG_OUT( "fmt = %d, hue = %d, sat = %d", fmt, tbar->hue[state], tbar->sat[state] );
 	if( tbar->hue[state] > 0 || tbar->sat[state] >= 0 )
 	{
 		ASImage *tmp_im = merge_layers (ASDefaultVisual, &layers[0], good_layers, tbar->width, tbar->height, ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT);		
@@ -1728,6 +1737,36 @@ LOCAL_DEBUG_OUT("back-try2(%p)", back );
     return res;
 }
 
+#ifdef TRACE_render_astbar
+#undef render_astbar
+Bool
+trace_render_astbar (ASTBarData * tbar, ASCanvas * pc, const char *file, int line)
+{
+    Bool res;
+    fprintf (stderr, "D>%s(%d):render_astbar(%p,%p)\n", file, line, tbar, pc);
+    res = render_astbar_int (tbar, pc, NULL, NULL);
+	fprintf (stderr,
+             "D>%s(%d):render_astbar(%p,%p) returned %d\n", file, line, tbar, pc, res);
+    if( tbar && res <= 0 )
+        fprintf( stderr, "D>%s(%d):render_astbar tbar data: state %lX, %ux%u%+d%+d, root %+d%+d, styles %p,%p, tiles_num %d, tiles %p, canvas %X\n" ,
+                 file, line, tbar->state, tbar->width, tbar->height, tbar->win_x, tbar->win_y, tbar->root_x, tbar->root_y, tbar->style[0], tbar->style[1], tbar->tiles_num, tbar->tiles, pc->canvas );
+    return res;
+}
+#else
+Bool
+render_astbar (ASTBarData * tbar, ASCanvas * pc)
+{
+	return render_astbar_int( tbar, pc, NULL, NULL );
+}
+#endif
+
+Bool
+render_astbar_cached_back (ASTBarData * tbar, ASCanvas * pc, ASImage **cache, ASCanvas *origin_canvas)
+{
+	return render_astbar_int( tbar, pc, cache, origin_canvas );
+}
+
+
 int
 check_astbar_point( ASTBarData *tbar, int root_x, int root_y )
 {
@@ -1819,7 +1858,7 @@ on_astbar_pointer_action( ASTBarData *tbar, int context, Bool leave, Bool pointe
 }
 
 void
-set_astbar_balloon( ASTBarData *tbar, int context, const char *text, unsigned long encoding )
+set_astbar_balloon2( ASTBarData *tbar, ASBalloonState *balloon_state, int context, const char *text, unsigned long encoding )
 {
     if( tbar != NULL )
     {
@@ -1845,7 +1884,7 @@ set_astbar_balloon( ASTBarData *tbar, int context, const char *text, unsigned lo
 												  );
                             }else
                             {
-                                bb->buttons[k].balloon = create_asballoon_with_text( tbar, text, encoding );
+                                bb->buttons[k].balloon = create_asballoon_with_text_for_state( balloon_state, tbar, text, encoding );
                                 LOCAL_DEBUG_OUT( "created balloon for tbar(%p)->ct(0x%X)->btn(%d)->enc(%ld)->text(%s)->balloon(%p)",
 								               tbar, context, k, encoding, text, bb->buttons[k].balloon );
                             }
@@ -1862,7 +1901,7 @@ set_astbar_balloon( ASTBarData *tbar, int context, const char *text, unsigned lo
                                                  tbar, context, text, tbar->balloon );
             }else
             {
-                tbar->balloon = create_asballoon_with_text( tbar, text, encoding );
+                tbar->balloon = create_asballoon_with_text_for_state( balloon_state, tbar, text, encoding );
                 LOCAL_DEBUG_OUT( "created tbar balloon for tbar(%p)->context(0x%X)->text(%s)->balloon(%p)",
                                  tbar, context, text, tbar->balloon );
             }
@@ -1871,4 +1910,10 @@ set_astbar_balloon( ASTBarData *tbar, int context, const char *text, unsigned lo
     }
 }
 
+void
+set_astbar_balloon( ASTBarData *tbar, int context, const char *text, unsigned long encoding )
+{
+    if( tbar != NULL )
+		set_astbar_balloon2( tbar, NULL, context, text, encoding );
+}
 
