@@ -45,6 +45,7 @@
 #include "screen.h"
 #include "freestor.h"
 #include "functions.h"
+#include "../libAfterImage/asimagexml.h"
 
 extern char  *_disabled_keyword;
 #if 0
@@ -78,30 +79,58 @@ FindTerm (SyntaxDef * syntax, int type, int id)
 
 
 /* debugging stuff */
-#ifdef DEBUG_PARSER
 void
-PrintFreeStorage (FreeStorageElem * storage)
+freestorage_print(char *myname, SyntaxDef *syntax, FreeStorageElem * storage, int level)
 {
-	static int    level = 0;
-
+	if( myname == NULL ) 
+		myname = MyName;
 	while (storage)
 	{
-		level++;
-		fprintf (stderr, "\n%d>Term's keyword: [%s]", level, storage->term->keyword);
-		fprintf (stderr, "\n%d>Data:", level);
+		char sub_terminator = ' ' ;
+		if( level > 0 ) 
+			if( syntax->terminator == '\n' || syntax->terminator == '\0' ) 
+				fprintf (stderr, "%*s", level*4, " ");
+
+		if( !get_flags( storage->term->flags, TF_NO_MYNAME_PREPENDING )  )
+		{	
+			fprintf (stderr, "*%s", myname);
+			if( storage->term->keyword[0] == '~' )
+				fputc( ' ', stderr );
+		}
+		fprintf (stderr, "%s", storage->term->keyword);
 		{
 			register int  i;
+			int token_count = storage->argc ; 
+			if( storage->term->sub_syntax ) 
+			{
+				token_count = GetTermUseTokensCount(storage->term);
+				if( get_flags( storage->term->flags, TF_NAMED|TF_INDEXED|TF_DIRECTION_INDEXED) )
+					++token_count;
+			}	 
 
-			for (i = 0; i < storage->argc; i++)
-				fprintf (stderr, "\n%d>  %d:\t[%s]", level, i, storage->argv[i]);
+			for (i = 0; i < token_count; i++)
+				fprintf (stderr, " %s", storage->argv[i]);
+			if( token_count == 0 && storage->sub && storage->term->sub_syntax )
+				fputc( ' ', stderr );
 		}
-		fprintf (stderr, "\n%d>Subterm: 0x%X", level, (int)(storage->sub));
-		PrintFreeStorage (storage->sub);
+		if( storage->sub && storage->term->sub_syntax ) 
+		{	
+			if( storage->term->sub_syntax->terminator == '\n' ) 
+				fputc( '\n', stderr );		
+ 			freestorage_print(myname, storage->term->sub_syntax, storage->sub, level+1);	
+			sub_terminator = storage->term->sub_syntax->file_terminator ;
+			if( sub_terminator == '\0' ) 
+				sub_terminator = '\n' ;
+		}
+		if( storage->next &&  syntax->terminator != '\0' && (sub_terminator == ' ' || sub_terminator != syntax->terminator ) ) 
+			fputc( syntax->terminator, stderr );
 		storage = storage->next;
-		level--;
 	}
+	if( syntax->file_terminator != '\0' ) 
+		fputc( syntax->file_terminator, stderr );
+	else
+		fputc( '\n', stderr );
 }
-#endif
 
 /* StringArray functionality */
 char        **
@@ -218,13 +247,9 @@ AddFreeStorageElem (SyntaxDef * syntax, FreeStorageElem ** tail, TermDef * pterm
 
 	if (pterm)
 	{
-		new_elem = (FreeStorageElem *) safemalloc (sizeof (FreeStorageElem));
+		new_elem = (FreeStorageElem *) safecalloc (1, sizeof (FreeStorageElem));
 		new_elem->term = pterm;
-		new_elem->argc = 0;
-		new_elem->argv = NULL;
-		new_elem->flags = 0;
 		new_elem->next = *tail;
-		new_elem->sub = NULL;
 		*tail = new_elem;
 	}
 	return new_elem;
@@ -238,14 +263,12 @@ DupFreeStorageElem (FreeStorageElem * source)
 
 	if (source)
 	{
-		new_elem = (FreeStorageElem *) safemalloc (sizeof (FreeStorageElem));
+		new_elem = (FreeStorageElem *) safecalloc (1, sizeof (FreeStorageElem));
 		new_elem->term = source->term;
 		new_elem->argc = source->argc;
 		/* duplicating argv here */
 		new_elem->argv = DupStringArray (source->argc, source->argv);
 		new_elem->flags = source->flags;
-		new_elem->next = NULL;
-		new_elem->sub = NULL;
 		if (new_elem->sub)
 		{
 			FreeStorageElem *psub, **pnew_sub = &(new_elem->sub);
@@ -436,8 +459,7 @@ free_storage2func (FreeStorageElem * stored, int *ppos)
 	if (stored->term == NULL)
 		return NULL;
 
-	pfunc = (struct FunctionData *)safemalloc (sizeof (struct FunctionData));
-	memset (pfunc, 0x00, sizeof (struct FunctionData));
+	pfunc = (struct FunctionData *)safecalloc (1, sizeof (struct FunctionData));
 
 	pfunc->func = stored->term->id;
 
@@ -913,7 +935,7 @@ FreeConfigItem (ConfigItem * item)
 		 break;
 
 	 case TT_BUTTON:
-#if 0
+#if 1
 		 destroy_asbutton (item->data.button, False);
 #endif
 		 break;
@@ -935,6 +957,54 @@ check_avail_args (FreeStorageElem * stored, int pos, int need)
 	return True;
 }
 
+int 
+direction2index(const char *ptr)	
+{
+	int index = 0;             
+    /* custom code to parse frame sides very fast :*/
+
+	if( ptr[0] == 'W' || ptr[0] == 'w' )
+        index = FR_W ;
+    else if( ptr[0] == 'E' || ptr[0] == 'e' )
+        index = FR_E ;
+    else if( ptr[0] == 'N' || ptr[0] == 'n' )
+    {
+        if( ptr[1] == 'W' || ptr[1] == 'w' )
+            index = FR_NW ;
+        else if( ptr[1] == 'E' || ptr[1] == 'e' )
+            index = FR_NE ;
+        else 
+        {
+            register int i = 0;
+            while( i < 5 && ptr[i] != '\0' ) ++i;
+            if( ptr[i] == 'W' || ptr[i] == 'w' )
+                index = FR_NW ;
+            else if( ptr[i] == 'E' || ptr[i] == 'e' )
+                index = FR_NE ;
+            else
+                index = FR_N ;
+        }
+    }else if( ptr[0] == 'S' || ptr[0] == 's' )
+    {
+        if( ptr[1] == 'W' || ptr[1] == 'w' )
+            index = FR_SW ;
+        else if( ptr[1] == 'E' || ptr[1] == 'e' )
+            index = FR_SE ;
+        else
+        {
+            register int i = 0;
+            while( i < 5 && ptr[i] != '\0' ) ++i;
+            if( ptr[i] == 'W' || ptr[i] == 'w' )
+                index = FR_SW ;
+            else if( ptr[i] == 'E' || ptr[i] == 'e' )
+                index = FR_SE ;
+            else
+                index = FR_S ;
+        }
+    }
+	return index;
+}
+
 int
 ReadConfigItem (ConfigItem * item, FreeStorageElem * stored)
 {
@@ -962,51 +1032,9 @@ ReadConfigItem (ConfigItem * item, FreeStorageElem * stored)
 			item->index = atoi (stored->argv[pos++]);
         }else if (get_flags(stored->term->flags, TF_DIRECTION_INDEXED))
         {
-            register char *ptr ;
 			if (!check_avail_args (stored, pos, 1))
 				return ret_code;
-            ptr = stored->argv[pos];
-            /* custom code to parse frame sides very fast :*/
-            if( ptr[0] == 'W' || ptr[0] == 'w' )
-                item->index = FR_W ;
-            else if( ptr[0] == 'E' || ptr[0] == 'e' )
-                item->index = FR_E ;
-            else if( ptr[0] == 'N' || ptr[0] == 'n' )
-            {
-                if( ptr[1] == 'W' || ptr[1] == 'w' )
-                    item->index = FR_NW ;
-                else if( ptr[1] == 'E' || ptr[1] == 'e' )
-                    item->index = FR_NE ;
-                else
-                {
-                    register int i = 0;
-                    while( i < 5 && ptr[i] != '\0' ) ++i;
-                    if( ptr[i] == 'W' || ptr[i] == 'w' )
-                        item->index = FR_NW ;
-                    else if( ptr[i] == 'E' || ptr[i] == 'e' )
-                        item->index = FR_NE ;
-                    else
-                        item->index = FR_N ;
-                }
-            }else if( ptr[0] == 'S' || ptr[0] == 's' )
-            {
-                if( ptr[1] == 'W' || ptr[1] == 'w' )
-                    item->index = FR_SW ;
-                else if( ptr[1] == 'E' || ptr[1] == 'e' )
-                    item->index = FR_SE ;
-                else
-                {
-                    register int i = 0;
-                    while( i < 5 && ptr[i] != '\0' ) ++i;
-                    if( ptr[i] == 'W' || ptr[i] == 'w' )
-                        item->index = FR_SW ;
-                    else if( ptr[i] == 'E' || ptr[i] == 'e' )
-                        item->index = FR_SE ;
-                    else
-                        item->index = FR_S ;
-                }
-            }
-            ++pos;
+            item->index = direction2index(stored->argv[pos++]);
         }
 		switch (item->type)
 		{
@@ -1019,12 +1047,15 @@ ReadConfigItem (ConfigItem * item, FreeStorageElem * stored)
 		 case TT_UINTEGER:
 			 if (!check_avail_args (stored, pos, 1))
 				 return 0;
+#if 0
 			 if( !isdigit(stored->argv[pos][0]) && stored->argv[pos][0] != '-' )
 			 {
 
 
 			 }else
 			 	item->data.integer = atol (stored->argv[pos]);
+#endif
+			 item->data.integer = parse_math(stored->argv[pos], NULL, 0);
 			 ++pos ;
 			 if (stored->term->type == TT_UINTEGER && item->data.integer < 0)
 			 {
@@ -1108,6 +1139,94 @@ ReadConfigItem (ConfigItem * item, FreeStorageElem * stored)
 	return 0;
 }
 
+Bool 
+ReadCompositeFlagsConfigItem( void *struct_ptr, ptrdiff_t flags_field_offset, FreeStorageElem * stored )
+{
+	if( struct_ptr && stored && flags_field_offset != 0 ) 
+	{
+		FreeStorageElem *pCurr = stored->sub ; 
+		ASFlagType result = 0 ; 
+
+		while( pCurr ) 
+		{
+			set_flags(result, pCurr->term->flags_on);
+			clear_flags(result, pCurr->term->flags_off);
+			pCurr = pCurr->next ; 
+		}
+		*((ASFlagType*)(struct_ptr+flags_field_offset)) = result ; 
+		return True;
+	}
+	return False;
+}
+
+
+Bool
+ReadConfigItemToStruct( void *struct_ptr, ptrdiff_t set_flags_offset, FreeStorageElem * stored ) 
+{
+	void *data_ptr = struct_ptr ;
+	Bool handled = False;
+
+	if( data_ptr == NULL || stored == NULL ) 
+		return False ;
+
+	if( stored->term->type == TT_FLAG && stored->term->sub_syntax )
+	{
+		handled = ReadCompositeFlagsConfigItem( struct_ptr, stored->term->struct_field_offset, stored );		
+	}else if( stored->term->struct_field_offset != 0 )	
+	{
+		int index = 0 ;
+		Bool handled = True;
+		ConfigItem item ;
+		data_ptr += stored->term->struct_field_offset ;
+		
+		item.memory = NULL ; 
+
+		if( !ReadConfigItem( &item, stored ) )
+			return False ;
+
+        if (get_flags(stored->term->flags, TF_INDEXED))
+			index =	item.index ;
+	
+		handled = True ;		
+		switch (item.type)
+		{
+		 case TT_FLAG: 		((Bool*) data_ptr)[index] = item.data.flag;		 break;
+		 case TT_INTEGER:	((int*)  data_ptr)[index] = item.data.integer;	 break;
+		 case TT_UINTEGER:	((unsigned int*)  data_ptr)[index] = item.data.integer;	 break;
+         case TT_BITLIST :	((int*)  data_ptr)[index] = item.data.integer;	 break;
+         case TT_OPTIONAL_PATHNAME:
+         case TT_COLOR:
+		 case TT_FONT:
+		 case TT_FILENAME:
+		 case TT_TEXT:
+		 case TT_PATHNAME:
+		 case TT_QUOTED_TEXT:
+						 	set_string(&(((char**)data_ptr)[index]), item.data.string);	 item.memory = NULL ; break;
+		 case TT_BOX:		((ASBox*)  data_ptr)[index] = item.data.box;	 break;
+		 case TT_GEOMETRY:	((ASGeometry*)  data_ptr)[index] = item.data.geometry;	 break;
+		 case TT_FUNCTION:  ((FunctionData**)data_ptr)[index] = item.data.function; item.memory = NULL ; break;
+		 case TT_BUTTON:	((ASButton**)data_ptr)[index] = item.data.button; item.memory = NULL; break;
+         case TT_CURSOR: 	((ASCursor**)data_ptr)[index] = item.data.cursor; item.memory = NULL ; break;
+
+		 case TT_INTARRAY:	
+         case TT_BINDING :	handled = False ; break;
+
+		 	default : handled = False ;
+        }
+		ReadConfigItem(&item, NULL);
+	}
+
+	if( handled && set_flags_offset > 0 ) 
+	{
+		ASFlagType *set_flags_ptr = struct_ptr+set_flags_offset ;
+
+		set_flags( *set_flags_ptr, stored->term->flags_on );
+		clear_flags( *set_flags_ptr, stored->term->flags_off );
+	}
+
+	return handled ;		
+}
+
 int
 ReadFlagItem (unsigned long *set_flags, unsigned long *flags, FreeStorageElem * stored, flag_options_xref * xref)
 {
@@ -1120,8 +1239,8 @@ ReadFlagItem (unsigned long *set_flags, unsigned long *flags, FreeStorageElem *
 			stored->sub != NULL )
 			return 0;
 
-		if (stored->argc > 1)
-			value = (atol (stored->argv[1]) > 0);
+		if (stored->argc >= 1)
+			value = (atol (stored->argv[0]) > 0);
 
 		while (xref->flag != 0)
 		{
@@ -1145,6 +1264,48 @@ ReadFlagItem (unsigned long *set_flags, unsigned long *flags, FreeStorageElem *
 	return 0;
 }
 
+int
+ReadFlagItemAuto (void *config_struct, ptrdiff_t default_set_flags_offset, FreeStorageElem * stored, flag_options_xref * xref)
+{
+	if (stored && xref )
+	{
+		Bool          value = True;
+
+        if (stored->term->type != TT_FLAG || 
+			get_flags (stored->term->flags, (TF_INDEXED|TF_DIRECTION_INDEXED)) ||
+			stored->sub != NULL )
+			return 0;
+
+		if (stored->argc >= 1)
+			value = (atol (stored->argv[0]) > 0);
+
+		while (xref->flag != 0)
+		{
+            if (xref->id_on == stored->term->id || xref->id_off == stored->term->id )
+            {
+				unsigned long *flags = (xref->flag_field_offset>0)?config_struct+xref->flag_field_offset:NULL;
+				unsigned long *set_flags  = (xref->set_flag_field_offset>0)?config_struct+xref->set_flag_field_offset:
+											(default_set_flags_offset>0)?config_struct+default_set_flags_offset:NULL ;
+                
+				if (set_flags)
+                    set_flags (*set_flags, xref->flag);
+                if (flags)
+                {
+                    if ((value && xref->id_on == stored->term->id) ||
+                        (!value && xref->id_off == stored->term->id))
+                        set_flags (*flags, xref->flag);
+                    else
+                        clear_flags (*flags, xref->flag);
+                }
+            }
+			xref++;
+		}
+        return 1;
+	}
+	return 0;
+}
+
+
 /****************************************************************************/
 /* D2F:  Datatype to FreeStorage conversion routines :			    */
 /****************************************************************************/
