@@ -42,9 +42,11 @@ typedef enum FunctionCode{
   F_POPUP,
   F_FUNCTION,
   F_CATEGORY,
+  F_CATEGORY_TREE,
   F_MINIPIXMAP,
   F_SMALL_MINIPIXMAP,
   F_LARGE_MINIPIXMAP,
+  F_DesktopEntry,
   F_EXEC,
   F_MODULE,
   F_ExecToolStart,
@@ -77,8 +79,10 @@ typedef enum FunctionCode{
   F_ENDFUNC,
   F_ENDPOPUP,
   F_TAKE_SCREENSHOT,
+  F_SET,
   F_Test,    /* for debugging purposes to be able to test new features before actually
               * enabling them for user */
+			 
   /* this functions require window as aparameter */
   F_WINDOW_FUNC_START,
   F_MOVE,
@@ -205,16 +209,21 @@ typedef struct MenuDataItem
     struct MenuDataItem  *next;  /* next menu item */
     struct MenuDataItem  *prev;  /* prev menu item */
 
+/* same flags are used for both MenuDataItem and MenuData */
+
 #define MD_Disabled        		(0x01<<0)
 #define MD_ScaleMinipixmapDown 	(0x01<<1)
 #define MD_ScaleMinipixmapUp  	(0x01<<2)
+#define MD_NameIsUTF8		  	(0x01<<16)
+#define MD_CommentIsUTF8	  	(0x01<<17)
 /* can't think of anything else atm - maybe add something later ? */
     ASFlagType            flags ;
     struct FunctionData  *fdata ;
     char                 *minipixmap ;         /* we always read filename from config !! */
     struct ASImage       *minipixmap_image ;
-    char *item;         /* the character string displayed on left */
-    char *item2;		/* the character string displayed on right */
+    char  *item;         /* the character string displayed on left */
+    char  *item2;		 /* the character string displayed on right */
+	char  *comment ;
 
     time_t                last_used_time ;
 }MenuDataItem;
@@ -222,13 +231,15 @@ typedef struct MenuDataItem
 typedef struct MenuData
 {
     unsigned long    magic;
-    char *name;         /* name of root */
-
     struct MenuDataItem *first; /* first item in menu */
     struct MenuDataItem *last;  /* last item in menu */
     short items_num;        /* number of items in the menu */
 
-	unsigned int    recent_items ;
+    ASFlagType            flags ; 
+    char  *name;         /* name of root */
+	char  *comment ;
+	int    recent_items ;   /*  negative value indicates that default, 
+								set in feel should be used */
 
 }MenuData;
 
@@ -299,7 +310,7 @@ Bool check_fdata_availability( FunctionData *fdata );
 MenuDataItem * add_menu_fdata_item( MenuData *menu, FunctionData *fdata, char *minipixmap, struct ASImage *img );
 MenuDataItem * menu_data_item_from_func (MenuData * menu, FunctionData * fdata, Bool do_check_availability);
 struct ASImage *check_scale_menu_pmap( struct ASImage *im, ASFlagType flags ); 
-void reload_menu_pmaps( MenuData *menu );
+void reload_menu_pmaps( MenuData *menu, Bool force  );
 
 /* usefull to qsort array of FunctionData pointers : */
 int compare_func_data_name(const void *a, const void *b);
