@@ -44,7 +44,7 @@ mystyle_free_back_icon( MyStyle *style )
             style->back_icon.alpha = None ;
         }
 
-        free_icon_resources (style->back_icon);
+        free_icon_resources (&(style->back_icon));
     }
     memset (&(style->back_icon), 0x00, sizeof (style->back_icon));
 }
@@ -811,7 +811,7 @@ mystyle_free_resources( MyStyle *style )
 {
     if( style->magic == MAGIC_MYSTYLE )
     {
-        if (style->user_flags & F_FONT)
+        if (get_flags( style->user_flags, F_FONT))
         {
             unload_font (&style->font);
         }
@@ -1003,25 +1003,20 @@ mystyle_find_or_default (const char *name)
  *   inherit if the parent style could go away before the child
  */
 void
-mystyle_merge_font( MyStyle *style, MyFont *font, Bool override, Bool copy)
+mystyle_merge_font( MyStyle *style, MyFont *font, Bool override)
 {
-    if ((override == True) && (style->user_flags & F_FONT))
+    if ( override && get_flags(style->user_flags, F_FONT))
+	{
         unload_font (&style->font);
-    if ((override == True) || !(style->set_flags & F_FONT))
+        clear_flags(style->user_flags, F_FONT);
+        clear_flags(style->set_flags, F_FONT);
+	}
+    if ( override || !get_flags(style->set_flags, F_FONT) )
     {
-        if (copy == False)
-        {
-            style->font = *font;
-            style->user_flags &= ~F_FONT;
-            style->inherit_flags |= F_FONT;
-        } else
-        {
-            if (load_font (font->name, &style->font) == True)
-            {
-                style->user_flags |= F_FONT;
-                style->inherit_flags &= ~F_FONT;
-            }
-        }
+		set_string( &(style->font.name), mystrdup(font->name));
+		style->font.as_font = dup_asfont( font->as_font );
+        set_flags(style->user_flags, F_FONT);
+        clear_flags(style->inherit_flags, F_FONT);
     }
 }
 
@@ -1031,7 +1026,7 @@ mystyle_merge_styles (MyStyle * parent, MyStyle * child, Bool override, Bool cop
 	if( parent == NULL || child == NULL ) 
 		return;
 	if (parent->set_flags & F_FONT)
-        mystyle_merge_font( child, &(parent->font), override, copy);
+        mystyle_merge_font( child, &(parent->font), override);
 
     if (parent->set_flags & F_TEXTSTYLE)
 	{
@@ -1367,9 +1362,10 @@ mystyle_inherit_font (MyStyle * style, MyFont * font)
 	 *       unloaded once */
 	if (style != NULL && !(style->set_flags & F_FONT))
 	{
-		style->font = *font;
-		style->inherit_flags |= F_FONT;
-		style->user_flags &= ~F_FONT;		   /* to prevent confusion */
+		set_string( &(style->font.name), font->name );
+		style->font.as_font = dup_asfont(font->as_font);
+		clear_flags(style->inherit_flags, F_FONT);
+		set_flags(style->user_flags, F_FONT);		   /* to prevent confusion */
 		style->set_flags = style->user_flags | style->inherit_flags;
 	}
 }
@@ -1491,6 +1487,10 @@ mystyle_draw_text_image (MyStyle * style, const char *text, unsigned long encodi
 
 	if (style && text)
 	{
+		/* load fonts on demand only - no need to waste memory if ain't never gonna use it ! */
+		if ( style->font.as_font == NULL )
+			load_font( NULL, &style->font );
+
 		if (style->font.as_font)
         {
 			ASTextAttributes attr = {ASTA_VERSION_1, ASTA_UseTabStops, AST_Plain, ASCT_Char, 8, 0, NULL, 0, ARGB32_White };
@@ -1521,7 +1521,26 @@ unsigned int
 mystyle_get_font_height (MyStyle * style)
 {
 	if (style)
+	{
+		if (style->font.as_font == NULL )
+			load_font( NULL, &style->font );
+
 		if (style->font.as_font)
 			return style->font.as_font->max_height;
+	}
 	return 1;
 }
+
+void
+mystyle_get_text_size (MyStyle * style, const char *text, unsigned int *width, unsigned int *height )
+{
+	if (style && text)
+	{
+		if (style->font.as_font == NULL )
+			load_font( NULL, &style->font );
+
+		if (style->font.as_font)
+			get_text_size( text, style->font.as_font, style->text_style, width, height );
+	}
+}
+
