@@ -29,9 +29,13 @@
 #include "docfile.h"
 
 const char *HTMLHeaderFormat = "<A name=\"page_top\"></A>\n"
+							   "<A href=\"preface.html\">Preface</A>&nbsp;&nbsp;"
+							   "<A href=\"visualselect.html\">Introduction</A>&nbsp;&nbsp;"
 							   "<A href=\"index.html\">%s</A>&nbsp;&nbsp;<A href=\"Glossary.html\">%s</A><p>\n" ;
 
 const char *HTMLHeaderFormatAPI = "<A name=\"page_top\"></A>\n"
+							   "<A href=\"../preface.html\">Preface</A>&nbsp;&nbsp;"
+							   "<A href=\"../visualselect.html\">Introduction</A>&nbsp;&nbsp;"
 							   "<A href=\"../index.html\">Main index</A>&nbsp;&nbsp;"
 							   "<A href=\"index.html\">%s</A>&nbsp;&nbsp;<A href=\"Glossary.html\">%s</A><p>\n" ;
 
@@ -86,7 +90,19 @@ write_doc_header( ASXMLInterpreterState *state )
 				fprintf( state->dest_fp, "<h1>%s</h1>\n", state->display_name );
 
 			break;
- 		case DocType_PHP :	
+ 		case DocType_PHP :
+		    if (TopicIndexName == NULL )
+		     {
+			fprintf( state->dest_fp, PHPXrefFormat, "graphics","bars","/desktop/bars/index","");
+			fprintf( state->dest_fp, PHPXrefFormat, "graphics","cursors","/desktop/cursors/index","");
+			fprintf( state->dest_fp, PHPXrefFormat, "graphics","fonts","/desktop/fonts/index","");
+			fprintf( state->dest_fp, PHPXrefFormat, "graphics","icons","/desktop/icons/index","");
+			fprintf( state->dest_fp, PHPXrefFormat, "graphics","tiles","/desktop/tiles/index","");
+			fprintf( state->dest_fp, PHPXrefFormat, "graphics","sounds","/desktop/sounds/index","");
+			fprintf( state->dest_fp, PHPXrefFormat, "graphics","buttons","/desktop/buttons/index","");
+		     }
+		    else
+		     {
 			fprintf( state->dest_fp, PHPXrefFormat, "documentation","Preface","", "" );
 			fprintf( state->dest_fp, PHPXrefFormat, "visualdoc","Introduction","visualselect", "" );
 
@@ -108,6 +124,7 @@ write_doc_header( ASXMLInterpreterState *state )
 
 			fprintf( state->dest_fp, PHPXrefFormat, "visualdoc","F.A.Q.","faq", "" );
 			fprintf( state->dest_fp, PHPXrefFormat, "visualdoc","Copyright","copyright", "" );
+		     }
 			fprintf( state->dest_fp, "<hr>\n" );
 			if( state->display_purpose[0] != '\0' )
 				fprintf( state->dest_fp, "<br><table width=100%%><tr>"
@@ -115,7 +132,7 @@ write_doc_header( ASXMLInterpreterState *state )
 				"<td width=50%%><FONT face=\"Tahoma, Arial, Verdana, Helvetica\" size=\"-1\">%s</FONT></td>"
 				"</tr></table><br><hr>\n", state->display_name, state->display_purpose );
 			else
-				fprintf( state->dest_fp, "<br><b>%s</b><br><br><hr>\n", state->display_name);
+				fprintf( state->dest_fp, "<br><b>%s</b><br><br>\n", state->display_name);
 			for( i = 0 ; i < DocClass_TopicIndex ; ++i ) 
 			{
 				if( get_flags( state->doc_class_mask, (0x01<<i)	) )
@@ -178,7 +195,7 @@ write_options_header( ASXMLInterpreterState *state )
 			break;
 		case DocType_HTML :
 			if( state->pre_options_size > 1024 )
-				fprintf( state->dest_fp,"<p><A href=\"index.html\">%s</A>&nbsp;&nbsp;<A href=\"Glossary.html\">%s</A>&nbsp;&nbsp;<A href=\"#page_top\">Back to Top</A><hr>\n", 
+				fprintf( state->dest_fp,"<p><A href=\"preface\">Preface</A>&nbsp;&nbsp;<A href=\"viusalselect.html\">Introduction</A>&nbsp;&nbsp;<A href=\"index.html\">%s</A>&nbsp;&nbsp;<A href=\"Glossary.html\">%s</A>&nbsp;&nbsp;<A href=\"#page_top\">Back to Top</A><hr>\n", 
 						 TopicIndexName, GlossaryName );
 			fprintf( state->dest_fp,"\n<UL><LI><A NAME=\"options\"></A><h3>CONFIGURATION OPTIONS :</h3>\n"
 							  		"<DL>\n");   
@@ -234,7 +251,7 @@ write_doc_footer( ASXMLInterpreterState *state )
 			break;
 		case DocType_HTML :
 			if( TopicIndexName != NULL )
-				fprintf( state->dest_fp,"<p><A href=\"index.html\">Topic index</A>&nbsp;&nbsp;<A href=\"Glossary.html\">Glossary</A>&nbsp;&nbsp;<A href=\"#page_top\">Back to Top</A>\n"); 
+				fprintf( state->dest_fp,"<p><A href=\"preface.html\">Preface</A>&nbsp;&nbsp;<A href=\"visualselect.html\">Introduction</A>&nbsp;&nbsp;<A href=\"index.html\">Topic index</A>&nbsp;&nbsp;<A href=\"Glossary.html\">Glossary</A>&nbsp;&nbsp;<A href=\"#page_top\">Back to Top</A>\n"); 
 			fprintf( state->dest_fp, 
 					"<hr>\n<p><FONT face=\"Verdana, Arial, Helvetica, sans-serif\" size=\"-2\">AfterStep version %s</a></FONT>\n"
 					"</body>\n</html>\n", VERSION );			   
