@@ -79,11 +79,13 @@ typedef struct {
 #define ASWT_StateMapped	(0x01<<0)
 #define ASWT_StateFocused	(0x01<<1)
 #define ASWT_AllDesks		(0x01<<2)
-#define ASWT_Transparent	(0x01<<3)
+#define ASWT_Transparent	 (0x01<<3)
 #define ASWT_ShutDownInProgress	(0x01<<4)
 #define ASWT_SkipTransients		(0x01<<5)
 #define ASWT_StateShaded		(0x01<<6)
 #define ASWT_StateSticky		(0x01<<7)
+#define ASWT_WantTransparent 	(0x01<<16) /* requested at command line */
+
 
 	ASFlagType flags ;
 
@@ -118,6 +120,7 @@ typedef struct {
 	
 	ASHashTable *unswallowed_apps ; 
 
+	unsigned long 		border_color;
 }ASWinTabsState ;
 
 ASWinTabsState WinTabsState = { 0 };
@@ -150,9 +153,11 @@ WinTabsConfig *Config = NULL ;
 char *pattern_override = NULL ;
 char *exclude_pattern_override = NULL ;
 char *title_override = NULL, *icon_title_override = NULL ;
+char *border_color_override = NULL ;
 
 void CheckConfigSanity(const char *pattern_override, const char *exclude_pattern_override, 
 					   const char *title_override, const char *icon_title_override);
+void SetWinTabsLook();
 void GetBaseOptions (const char *filename);
 void GetOptions (const char *filename);
 void HandleEvents();
@@ -186,6 +191,7 @@ void update_tabs_state();
 void register_unswallowed_client( Window client );
 Bool check_unswallowed_client( Window client ); 
 void delete_tab( int index );
+void set_frame_background( ASWinTab *aswt );
 
 /* above function may also return : */
 #define BANNER_TAB_INDEX -1		   
@@ -194,7 +200,7 @@ void delete_tab( int index );
 
 void DeadPipe(int);
 
-CommandLineOpts WinTabs_cmdl_options[8] =
+CommandLineOpts WinTabs_cmdl_options[9] =
 {
 	{NULL, "pattern","Overrides module's inclusion pattern", NULL,
 	 handler_set_string, &pattern_override, 0, CMO_HasArgs },
@@ -214,8 +220,11 @@ CommandLineOpts WinTabs_cmdl_options[8] =
 	{NULL, "skip-transients","Disregard any transient window(dialog)", NULL, handler_set_flag,
 	 &(WinTabsState.flags), ASWT_SkipTransients, 0 },
     
-	{"tr", "transparent","keep window-background transparent", NULL, handler_set_flag,
-	 &(WinTabsState.flags), ASWT_Transparent, 0 },
+	{"tr", "transparent","keep window border transparent", NULL, handler_set_flag,
+	 &(WinTabsState.flags), ASWT_WantTransparent, 0 },
+
+	{"bc", "border-color","use color to fill border around windows", NULL, handler_set_string,
+	 &border_color_override, 0, CMO_HasArgs },
 
 	{NULL, NULL, NULL, NULL, NULL, NULL, 0, 0 }
 };
@@ -260,33 +269,32 @@ main( int argc, char **argv )
 	LinkAfterStepConfig();
 
     set_signal_handler( SIGSEGV );
-	set_flags( WinTabsState.flags, ASWT_Transparent );  /* default */
-    
 
-	
-for( i = 1 ; i< argc ; ++i)
- {
-	 LOCAL_DEBUG_OUT( "argv[%d] = \"%s\", original argv[%d] = \"%s\"", i, argv[i], i, MyArgs.saved_argv[i]);	  
-	 if( argv[i] != NULL )
-	 { 	
-		 if( ( opt = match_command_line_opt( &(argv[i][0]), WinTabs_cmdl_options ) ) < 0)
-			 continue;
-		 
-		 /* command-line-option 'opt' has been matched */
-		 
-		 if( get_flags ( WinTabs_cmdl_options[opt].flags, CMO_HasArgs) )
-			 if( ++i >= argc)
-				 continue;
-		 
-		 WinTabs_cmdl_options[opt].handler( argv[i], WinTabs_cmdl_options[opt].trg,
-						    WinTabs_cmdl_options[opt].param);
-		 
-	 }
- }
+	for( i = 1 ; i< argc ; ++i)
+	{
+		LOCAL_DEBUG_OUT( "argv[%d] = \"%s\", original argv[%d] = \"%s\"", i, argv[i], i, MyArgs.saved_argv[i]);	  
+		if( argv[i] != NULL )
+		{ 	
+			if( ( opt = match_command_line_opt( &(argv[i][0]), WinTabs_cmdl_options ) ) < 0)
+			 	continue;
+
+			/* command-line-option 'opt' has been matched */
+
+			if( get_flags ( WinTabs_cmdl_options[opt].flags, CMO_HasArgs) )
+			 	if( ++i >= argc)
+				 	continue;
+
+			WinTabs_cmdl_options[opt].handler( argv[i], WinTabs_cmdl_options[opt].trg,
+							WinTabs_cmdl_options[opt].param);
+
+		}
+	}
 
     ConnectX( ASDefaultScr, EnterWindowMask );
     ConnectAfterStep ( WINTABS_MESSAGE_MASK, 0 );               /* no AfterStep */
-	
+
+	WinTabsState.border_color = Scr.asv->black_pixel; /* default */
+		
 	signal (SIGPIPE, OnDisconnect);
     
 	signal (SIGTERM, DeadPipe);
@@ -298,6 +306,7 @@ for( i = 1 ; i< argc ; ++i)
 	LoadColorScheme();
 	LoadConfig ("wintabs", GetOptions);
     CheckConfigSanity(pattern_override, exclude_pattern_override, title_override, icon_title_override);
+	SetWinTabsLook();
 
 	SendInfo ("Send_WindowList", 0);
 
@@ -305,6 +314,7 @@ for( i = 1 ; i< argc ; ++i)
 	WinTabsState.tabs = create_asvector( sizeof(ASWinTab) );
 
     WinTabsState.main_window = make_wintabs_window();
+	set_frame_background(NULL);
     WinTabsState.main_canvas = create_ascanvas_container( WinTabsState.main_window );
     WinTabsState.tabs_window = make_tabs_window( WinTabsState.main_window );
     WinTabsState.tabs_canvas = create_ascanvas( WinTabsState.tabs_window );
@@ -427,13 +437,6 @@ void
 CheckConfigSanity(const char *pattern_override, const char *exclude_pattern_override, 
 				  const char *title_override, const char *icon_title_override)
 {
-    
-	int i ;
-    char *default_style = safemalloc( 1+strlen(MyName)+1);
-	
-	default_style[0] = '*' ;
-	strcpy( &(default_style[1]), MyName );
-
     if( Config == NULL )
         Config = CreateWinTabsConfig ();
 
@@ -501,6 +504,17 @@ LOCAL_DEBUG_OUT( "exclude_pattern = \"%s\"", Config->exclude_pattern );
     Config->anchor_y = get_flags( Config->geometry.flags, YValue )?Config->geometry.y:0;
     if( get_flags(Config->geometry.flags, YNegative) )
         Config->anchor_y += Scr.MyDisplayHeight ;
+}
+
+void
+SetWinTabsLook()
+{
+	int i ;
+    char *default_style = safemalloc( 1+strlen(MyName)+1);
+	ARGB32 border_color = 0; 
+	
+	default_style[0] = '*' ;
+	strcpy( &(default_style[1]), MyName );
 
 	if( WinTabsState.tbar_props == NULL ) 
 	    retrieve_wintabs_astbar_props();
@@ -527,18 +541,25 @@ LOCAL_DEBUG_OUT( "exclude_pattern = \"%s\"", Config->exclude_pattern );
 		}
     }
     free( default_style );
+	if( get_flags( WinTabsState.flags, ASWT_WantTransparent ) )
+		set_flags( WinTabsState.flags, ASWT_Transparent );
+	else
+	{
+		clear_flags( WinTabsState.flags, ASWT_Transparent );
+		border_color = Scr.Look.MSWindow[BACK_FOCUSED]->colors.back;
+		if( border_color_override != NULL ) 
+			parse_argb_color( border_color_override, &border_color );
+		else if( TransparentMS(Scr.Look.MSWindow[BACK_FOCUSED]) )
+			set_flags( WinTabsState.flags, ASWT_Transparent );
+		ARGB2PIXEL(Scr.asv,border_color,&WinTabsState.border_color);
+		XSetForeground(dpy, Scr.DrawGC, WinTabsState.border_color);
+	}
+		
 
-#if defined(LOCAL_DEBUG) && !defined(NO_DEBUG_OUTPUT)
-    PrintWinTabsConfig (Config);
-    Print_balloonConfig ( Config->balloon_conf );
-#endif
-	
 	balloon_config2look( &(Scr.Look), NULL, Config->balloon_conf, "*WinTabsBalloon" );
     set_balloon_look( Scr.Look.balloon_look );
-
 }
 
-
 void
 GetBaseOptions (const char *filename)
 {
@@ -772,6 +793,8 @@ DispatchEvent (ASEvent * event)
                         {
                             handle_canvas_config( tabs[i].client_canvas );
                             send_swallowed_configure_notify(&(tabs[i]));
+							if (get_flags (WinTabsState.flags, ASWT_Transparent))
+								XClearArea (dpy, tabs[i].frame_canvas->w, 0, 0, 0, 0, True);
                         }    
                     }    
                 }else if( event->w == WinTabsState.tabs_window ) 
@@ -834,10 +857,10 @@ DispatchEvent (ASEvent * event)
         case LeaveNotify :
             break;
 		case FocusIn :
-			set_flags(WinTabsState.flags, ASWT_StateFocused );
-			update_focus();
+  			set_flags(WinTabsState.flags, ASWT_StateFocused );
 		    break ;
-		case FocusOut : clear_flags(WinTabsState.flags, ASWT_StateFocused );
+		case FocusOut : 
+        clear_flags(WinTabsState.flags, ASWT_StateFocused );
 		    break ;
         case MotionNotify :
             if( pointer_tab >= 0 && (event->x.xmotion.state&AllButtonMask) != 0) 
@@ -853,19 +876,17 @@ DispatchEvent (ASEvent * event)
             break;
 		case Expose :
 #if 1
+			if ( !get_flags( WinTabsState.flags, ASWT_Transparent ) )
 			{	  
                 int i = PVECTOR_USED(WinTabsState.tabs);
                 ASWinTab *tabs = PVECTOR_HEAD( ASWinTab, WinTabsState.tabs );
-				while( ASCheckTypedWindowEvent( event->w, Expose, &(event->x) ) );
 				while( --i >= 0 ) 
-				{	
 					if( event->w == tabs[i].frame_canvas->w ) 
 					{
-						XSetBackground(dpy, Scr.DrawGC, Scr.asv->black_pixel);
-						XFillRectangle(dpy, tabs[i].frame_canvas->w, Scr.DrawGC, 0, 0, tabs[i].frame_canvas->width, tabs[i].frame_canvas->height);
+						XExposeEvent *xexp = &(event->x.xexpose);
+						XFillRectangle(dpy, tabs[i].frame_canvas->w, Scr.DrawGC, xexp->x, xexp->y, xexp->width, xexp->height);
 						break;	
 					}	 
-				}
 			}
 #endif
 		    break ;
@@ -874,6 +895,11 @@ DispatchEvent (ASEvent * event)
 			{	
 				if( event->x.xclient.data.l[0] == _XA_WM_DELETE_WINDOW )
 			    	DeadPipe(0);
+				else if( event->x.xclient.data.l[0] == _XA_WM_TAKE_FOCUS )
+				{
+					set_flags(WinTabsState.flags, ASWT_StateFocused);
+					update_focus();
+				}
 			}
 	        break;
 	    case PropertyNotify:
@@ -901,10 +927,14 @@ DispatchEvent (ASEvent * event)
                 	LOCAL_DEBUG_OUT( "AS Styles updated!%s","");
 					mystyle_list_destroy_all(&(Scr.Look.styles_list));
 					LoadColorScheme();
-					CheckConfigSanity(NULL, NULL, NULL, NULL);
+					SetWinTabsLook();
 					/* now we need to update everything */
-                	while( --i >= 0 ) 
+					set_frame_background(NULL);
+	            	while( --i >= 0 ) 
+					{
+						set_frame_background(&(tabs[i]));
                     	set_tab_look( &(tabs[i]), False);
+					}
 					set_tab_look( &(WinTabsState.banner), True);
                 	rearrange_tabs(False );
             	}else if( event->x.xproperty.atom == _AS_TBAR_PROPS )
@@ -1060,6 +1090,19 @@ show_banner_buttons()
 
 }	  
 
+void
+set_frame_background( ASWinTab *aswt )
+{
+	Window w = aswt?aswt->frame_canvas->w:WinTabsState.main_window;
+	if( get_flags( WinTabsState.flags, ASWT_Transparent ) )
+	{
+		XSetWindowBackgroundPixmap( dpy, w, ParentRelative);
+		LOCAL_DEBUG_OUT( "Is transparent %s", "" );
+	}/*else
+		XSetWindowBackground( dpy, w, WinTabsState.border_color);*/
+	XClearArea (dpy, w, 0, 0, 0, 0, True); /* generate expose events ! */
+}
+
 Window
 make_wintabs_window()
 {
@@ -1069,11 +1112,10 @@ make_wintabs_window()
 	int x, y ;
     unsigned int width = max(Config->geometry.width,1);
     unsigned int height = max(Config->geometry.height,1);
-    XSetWindowAttributes attributes;
 	char *iconic_name ; 
+	
 
 	memset( &extwm_hints, 0x00, sizeof(extwm_hints));
-    attributes.background_pixmap = ParentRelative;
     switch( Config->gravity )
 	{
 		case NorthEastGravity :
@@ -1095,7 +1137,8 @@ make_wintabs_window()
 			break;
 	}
     LOCAL_DEBUG_OUT( "creating main window with geometry %dx%d%+d%+d", width, height, x, y );
-    w = create_visual_window( Scr.asv, Scr.Root, x, y, 1, 1, 0, InputOutput, CWBackPixmap, &attributes);
+
+    w = create_visual_window( Scr.asv, Scr.Root, x, y, 1, 1, 0, InputOutput, 0, NULL);
     LOCAL_DEBUG_OUT( "main window created with Id %lX", w);
 
 	iconic_name = Config->icon_title ; 
@@ -1118,7 +1161,7 @@ make_wintabs_window()
 	extwm_hints.type_flags = EXTWM_TypeASModule|EXTWM_TypeNormal ;
 	
 
-	set_client_hints( w, NULL, &shints, AS_DoesWmDeleteWindow, &extwm_hints );
+	set_client_hints( w, NULL, &shints, AS_DoesWmDeleteWindow|AS_DoesWmTakeFocus, &extwm_hints );
 	set_client_cmd (w);
 
     /* we will need to wait for PropertyNotify event indicating transition
@@ -1155,19 +1198,8 @@ make_frame_window( Window parent )
 	static XSetWindowAttributes attr ;
 	ASFlagType attr_mask ;
     Window w ;
-	attr.event_mask = SubstructureRedirectMask|FocusChangeMask|KeyPressMask ;
+	attr.event_mask = SubstructureRedirectMask|FocusChangeMask|KeyPressMask|ExposureMask ;
 	attr_mask = CWEventMask ;
-	if( get_flags( WinTabsState.flags, ASWT_Transparent ) )
-	{
-		attr.background_pixmap = ParentRelative;
-		attr_mask |= CWBackPixmap ;
-		attr.event_mask |= ExposureMask ;
-		LOCAL_DEBUG_OUT( "Is transparent %s", "" );
-	}else
-	{		 
-		attr.background_pixel = Scr.asv->white_pixel;
-		attr_mask |= CWBackPixel ;
-	}
     w = create_visual_window( Scr.asv, parent, 0, 0, WinTabsState.win_width, WinTabsState.win_height, 0, InputOutput, attr_mask, &attr );
 	XGrabKey( dpy, WINTABS_SWITCH_KEYCODE, WINTABS_SWITCH_MOD, w, True, GrabModeAsync, GrabModeAsync);
     return w;
@@ -1179,6 +1211,8 @@ make_frame_window( Window parent )
 void
 set_tab_look( ASWinTab *aswt, Bool no_bevel )
 {
+	set_astbar_style_ptr (aswt->bar, -1, Scr.Look.MSWindow[BACK_UNFOCUSED]);
+	set_astbar_style_ptr (aswt->bar, BAR_STATE_FOCUSED,   Scr.Look.MSWindow[BACK_FOCUSED]);
 	if( no_bevel ) 
 	{
 		set_astbar_hilite( aswt->bar, BAR_STATE_UNFOCUSED, NO_HILITE|NO_HILITE_OUTLINE );
@@ -1196,8 +1230,6 @@ set_tab_look( ASWinTab *aswt, Bool no_bevel )
 	}
 	set_astbar_composition_method( aswt->bar, BAR_STATE_UNFOCUSED, Config->ucm );
 	set_astbar_composition_method( aswt->bar, BAR_STATE_FOCUSED,   Config->fcm );
-	set_astbar_style_ptr (aswt->bar, BAR_STATE_UNFOCUSED, Scr.Look.MSWindow[BACK_UNFOCUSED]);
-	set_astbar_style_ptr (aswt->bar, BAR_STATE_FOCUSED,   Scr.Look.MSWindow[BACK_FOCUSED]);
 }
 
 void
@@ -1268,7 +1300,8 @@ delete_tab( int index )
         WinTabsState.pressed_tab = -1 ;
     }
     destroy_astbar( &(tabs[index].bar) );
-    XSelectInput (dpy, tabs[index].client, NoEventMask);
+	/* we still want to receive DestroyNotify events */
+    XSelectInput (dpy, tabs[index].client, StructureNotifyMask);
     destroy_ascanvas( &(tabs[index].client_canvas) );
 	XDestroyWindow(dpy, tabs[index].frame_canvas->w );
 	destroy_ascanvas( &(tabs[index].frame_canvas) );
@@ -1350,11 +1383,6 @@ moveresize_client( ASWinTab *aswt, int x, int y, int width, int height )
 
 	moveresize_canvas( aswt->frame_canvas, 0, y, frame_width, frame_height );    
     moveresize_canvas( aswt->client_canvas, (frame_width - width)/2, (frame_height - height)/2, width, height );
-	if( !get_flags( WinTabsState.flags, ASWT_Transparent ) )
-	{	
-		XSetWindowBackground( dpy, aswt->frame_canvas->w, Scr.asv->black_pixel );
-		XClearWindow( dpy, aswt->frame_canvas->w );
-	}
 }
 
 void
@@ -1401,15 +1429,21 @@ rearrange_tabs( Bool dont_resize_window )
 	{	
 		max_y = tab_height ;
 		if( !dont_resize_window )
+		{
+			LOCAL_DEBUG_OUT( "resizing main canvas to %dx%d", x, max_y ) ;
 			if( resize_canvas( WinTabsState.main_canvas, x, max_y ) != 0 ) 
 				return;
+		}
 		max_x = x = mc->width ; 
 		tab_height = mc->height ;
 	}else
 	{	
 		if( !dont_resize_window )
+		{
+			LOCAL_DEBUG_OUT( "resizing main canvas to %dx%d", WinTabsState.win_width, WinTabsState.win_height ) ;
 			if( resize_canvas( WinTabsState.main_canvas, WinTabsState.win_width, WinTabsState.win_height ) != 0 ) 
 				return;
+		}
 		max_x = WinTabsState.win_width ; 
 		max_y = WinTabsState.win_height ;
 
@@ -1584,6 +1618,7 @@ do_swallow_window( ASWindowData *wd )
     char *name = NULL ;
 	INT32 encoding ;
 	ASWinTab *aswt = NULL ;
+	int gravity = NorthWestGravity ;
 
 	if( wd->client == WinTabsState.main_window )
 		return;
@@ -1628,13 +1663,42 @@ do_swallow_window( ASWindowData *wd )
 		return ;
 	}
 
+	/* if the window was previously unswallowed by us - we may get UnMapNotify
+	   after reparenting, which will make us immidiately unswallow this client.
+	   To prevent this :  */
+    XSelectInput (dpy, wd->client, NoEventMask);
+
     /* first thing - we reparent window and its icon if there is any */
     nc = aswt->client_canvas = create_ascanvas_container( wd->client );
 	aswt->frame_canvas = create_ascanvas_container( make_frame_window(WinTabsState.main_window) );
-	aswt->swallow_location.x = nc->root_x ; 
-	aswt->swallow_location.y = nc->root_y ; 
+	set_frame_background(aswt);
+
 	aswt->swallow_location.width = nc->width ; 
 	aswt->swallow_location.height = nc->height ; 
+	if( get_flags( wd->hints.flags, PWinGravity )  )
+		gravity = wd->hints.win_gravity ;
+#if 0	
+	aswt->swallow_location.x = nc->root_x ; 
+	aswt->swallow_location.y = nc->root_y ; 
+#else
+	if( gravity == CenterGravity ) 
+	{
+		aswt->swallow_location.x = (wd->frame_rect.x + (int)wd->frame_rect.width/2) - (int)nc->width/2 ; 
+		aswt->swallow_location.y = (wd->frame_rect.y + (int)wd->frame_rect.height/2) - (int)nc->height/2 ; 
+	}else if( gravity == StaticGravity ) 
+	{
+		aswt->swallow_location.x = nc->root_x ; 
+		aswt->swallow_location.y = nc->root_y ; 
+	}else
+	{
+		aswt->swallow_location.x = wd->frame_rect.x ;
+		aswt->swallow_location.y = wd->frame_rect.y ;
+		if( gravity == NorthEastGravity || gravity == SouthEastGravity ) 
+			aswt->swallow_location.x += (int)wd->frame_rect.width - (int)nc->width ;
+		if( gravity == SouthWestGravity || gravity == SouthEastGravity ) 
+			aswt->swallow_location.y += (int)wd->frame_rect.height - (int)nc->height ;
+	}
+#endif	
 	aswt->hints = wd->hints ;
 	aswt->hints.flags = wd->flags ;
 	check_wm_protocols( aswt );
@@ -1766,12 +1830,15 @@ check_swallow_window( ASWindowData *wd )
 	do_swallow_window( wd );
 }
 
+void unregister_client( Window client );
 
 void 
 on_destroy_notify(Window w)
 {
     ASWinTab *tabs = PVECTOR_HEAD(ASWinTab,WinTabsState.tabs);
     int i = PVECTOR_USED(WinTabsState.tabs) ;
+
+	unregister_client(w);
     while( --i >= 0 ) 
         if( tabs[i].client == w ) 
         {
@@ -1984,6 +2051,13 @@ register_unswallowed_client( Window client )
 	add_hash_item( WinTabsState.unswallowed_apps, AS_HASHABLE(client), NULL );
 }
 
+void 
+unregister_client( Window client )
+{
+	if( WinTabsState.unswallowed_apps) 
+		remove_hash_item( WinTabsState.unswallowed_apps, AS_HASHABLE(client), NULL, False);
+}
+
 
 Bool 
 check_unswallowed_client( Window client ) 
