@@ -46,6 +46,9 @@ ASVector     *Modules       = NULL;
 int           Module_fd     = 0;
 int           Module_npipes = 8;
 
+ASBalloonState *MenuBalloons = NULL ; 
+ASBalloonState *TitlebarBalloons = NULL ; 
+
 /**************************************************************************/
 void          SetupScreen();
 void          CleanupScreen();
@@ -402,6 +405,9 @@ SetupScreen()
     Scr.Windows = init_aswindow_list();
 
 	XSelectInput (dpy, Scr.Root, AS_ROOT_EVENT_MASK);
+
+	MenuBalloons = create_balloon_state(); 
+	TitlebarBalloons = create_balloon_state(); 
 }
 
 void
@@ -416,6 +422,9 @@ CleanupScreen()
         ungrab_server();
     }
 
+	destroy_balloon_state(&TitlebarBalloons); 
+	destroy_balloon_state(&MenuBalloons); 
+
 	release_all_old_background( True );
 
     DestroyManagementWindows();
@@ -725,15 +734,15 @@ LOCAL_DEBUG_CALLER_OUT( "%s restart, cmd=\"%s\"", restart?"Do":"Don't", command?
 		DestroyPendingFunctionsQueue();
 		DestroyCategories();
 
-	    balloon_init (True);
+	    cleanup_default_balloons();
 		destroy_asdatabase();
     	destroy_assession( Session );
 		destroy_asenvironment( &Environment );
     	/* pixmap references */
     	build_xpm_colormap (NULL);
 
-        restack_window_list(INVALID_DESK, True);
-
+		free_scratch_ids_vector();
+		free_scratch_layers_vector();		
         clientprops_cleanup ();
         wmprops_cleanup ();
 
