@@ -259,7 +259,7 @@ do_anim_shrink( void *vdata )
 		XDestroyWindow (dpy, data->cover);
 		free( data );
 	}else
-		timer_new (20, do_anim_shrink, data);	   	 
+		timer_new (10, do_anim_shrink, data);	   	 
 }
 
 void
@@ -405,6 +405,9 @@ void cover_desktop()
 	Window        w;
     XGCValues       gcvalue;
 
+	if( get_flags( Scr.Feel.flags, DontCoverDesktop ) )
+		return;
+		
     ++_as_desktop_cover_recursion ;
 
     if( _as_desktop_cover != None ) 
@@ -521,6 +524,7 @@ display_progress( Bool new_line, const char *msg_format, ... )
         y = Scr.MyDisplayHeight/5 + height * _as_progress_line ;
         _as_progress_cursor += XTextWidth( _as_desktop_cover_xfs, &buffer[0], len) + 10;
         XDrawString (dpy, _as_desktop_cover, _as_desktop_cover_gc, x, y, &buffer[0], len);
+		ASSync(False);		
     }        
 }    
 
