@@ -861,6 +861,61 @@ update_transp_iter_func(void *data, void *aux_data)
     return True;
 }
 
+static Bool 
+check_wm_hints_changed( ASWindow *asw )
+{
+	unsigned char *ptr = (unsigned char*)&(asw->saved_wm_hints) ; 
+	XWMHints *tmp = XGetWMHints (dpy, asw->w);
+	Bool changed = False ; 
+	if( tmp == NULL )
+	{
+		int i ;
+		for( i = 0 ; i < sizeof(XWMHints); ++i ) 
+			if( ptr[i] != 0 ) 
+				return True ;
+	}else
+	{
+		changed = (memcmp( tmp, ptr, sizeof(XWMHints) ) != 0);
+		XFree( tmp );
+	}
+	return changed ;
+}
+
+static Bool 
+check_wm_normal_hints_changed( ASWindow *asw )
+{
+	unsigned char *ptr = (unsigned char*)&(asw->saved_wm_normal_hints) ; 
+	XSizeHints *tmp = XAllocSizeHints();
+	unsigned char *ptr2 = (unsigned char *)tmp ;
+	Bool changed = False ; 
+	long unused ; 
+	int i ;
+
+	if( XGetWMNormalHints (dpy, asw->w, tmp, &unused) == 0 )
+	{
+		for( i = 0 ; i < sizeof(XSizeHints); ++i ) 
+			if( ptr[i] != 0 ) 
+			{	changed = True ; break ; }
+	}else
+		for( i = 0 ; i < sizeof(XSizeHints); ++i ) 
+			if( ptr[i] != ptr2[i] ) 
+			{	changed = True ; break ; }
+
+#if defined(LOCAL_DEBUG) && !defined(NO_DEBUG_OUTPUT)
+	if( changed ) 
+	{
+		LOCAL_DEBUG_OUT( "normal hints differ at offset %d", i ); 
+		LOCAL_DEBUG_OUT( "Old hints : %s","" ); 
+		print_wm_normal_hints  ( NULL, NULL, (XSizeHints*)ptr );
+		LOCAL_DEBUG_OUT( "New hints : %s","" ); 
+		print_wm_normal_hints  ( NULL, NULL, tmp );
+	}
+#endif	
+	XFree( tmp );
+	return changed ;
+}
+
+
 void
 HandlePropertyNotify (ASEvent *event)
 {
@@ -956,6 +1011,20 @@ HandlePropertyNotify (ASEvent *event)
 		if( old_name )
 			free( old_name );
     	LOCAL_DEBUG_OUT( "hints flags = %lX, ShortLived ? %lX ", asw->hints->flags, ASWIN_HFLAGS( asw, AS_ShortLived ) );
+#if (defined(LOCAL_DEBUG)||defined(DEBUG)) && defined(DEBUG_ALLOCS)
+		{
+			static time_t old_t = 0 ; 
+			time_t t = time(NULL);
+			if( old_t < t ) 
+			{
+				char fname[256] ; 
+				sprintf( fname, "afterstep.allocs.name_change.%lu.log", t ); 
+//		    	spool_unfreed_mem( fname, NULL );
+				old_t = t ;
+			}
+		}
+#endif
+
 	/* otherwise we should check if this is the status property that we change ourselves : */
     }else if( atom == XA_WM_COMMAND || atom == XA_WM_CLIENT_MACHINE )
 	{
@@ -964,6 +1033,22 @@ HandlePropertyNotify (ASEvent *event)
 	{	
 	 	if( event->context == C_WINDOW && event->w != asw->frame ) 
 			on_window_opacity_changed( asw );
+	}else if( atom == XA_WM_HINTS )
+	{
+		if( check_wm_hints_changed( asw ) ) 
+	        on_window_hints_changed( asw );
+		else
+		{
+    		LOCAL_DEBUG_OUT( "ignoring WM_HINTS change - data is the same%s","" );
+		}		
+	}else if( atom == XA_WM_NORMAL_HINTS )
+	{
+		if( check_wm_normal_hints_changed( asw ) ) 
+	        on_window_hints_changed( asw );
+		else
+		{
+    		LOCAL_DEBUG_OUT( "ignoring WM_NORMAL_HINTS change - data is the same%s","" );
+		}		
 	}else if( NeedToTrackPropChanges(atom) )
         on_window_hints_changed( asw );
 
@@ -971,6 +1056,52 @@ HandlePropertyNotify (ASEvent *event)
 }
 
 
+#ifdef NAME_CHANGE_TEST
+/* our status */
+ASFlagType AfterStepState = 0; /* default status */
+
+/* Config : */
+ASVector     *Modules       = NULL;
+int           Module_fd     = 0;
+int           Module_npipes = 8;
+
+ASBalloonState *MenuBalloons = NULL ; 
+ASBalloonState *TitlebarBalloons = NULL ; 
+
+void
+Done (Bool restart, char *command )
+{
+	    exit(0);
+}
+
+int
+main(int argc, char **argv)
+{
+	Window w ;
+	int i = 0;
+	char window_name[512] ; 
+
+    InitMyApp ("test_title_change", argc, argv, NULL, NULL, 0 );
+	
+	ConnectX(ASDefaultScr, 0);
+	
+	w =  create_visual_window( Scr.asv, Scr.Root, 0, 0, 300, 300, 0, InputOutput, 0, NULL);
+	
+	sprintf( window_name, "test title change - iter %d", i ) ;
+    set_client_names( w, window_name, window_name, "test", "test_title_change" );
+	
+	XSetWindowBackground(dpy, w, 0 );
+	XMapWindow( dpy, w );
+	while(1)
+	{
+		ASSync(False);
+		sleep_a_millisec(100);
+		sprintf( window_name, "test title change - iter %d", ++i ) ;
+	    set_client_names( w, window_name, window_name, "test", "test_title_change" );
+	}	
+	return 0;
+}
+#endif
 /***********************************************************************
  *
  *  Procedure:
