@@ -30,6 +30,8 @@
 
 #include "asinternals.h"
 
+#include <stdlib.h>
+#include <unistd.h>
 #include "../../libAfterStep/wmprops.h"
 #include "../../libAfterStep/session.h"
 #include "../../libAfterStep/moveresize.h"
@@ -189,7 +191,7 @@ viewport_aswindow_iter_func( void *data, void *aux_data )
         asw->status->viewport_x = Scr.Vx ;
         asw->status->viewport_y = Scr.Vy ;
         if( !ASWIN_GET_FLAGS(asw,AS_Sticky) )
-            on_window_status_changed( asw, True, True );
+            on_window_status_changed( asw, True );
 
         if( ASWIN_GET_FLAGS(asw,AS_Iconic) && get_flags(Scr.Feel.flags, StickyIcons))
         {   /* we must update let all the modules know that icon viewport has changed
@@ -288,7 +290,7 @@ deskviewport_aswindow_iter_func(void *data, void *aux_data)
         	quietly_reparent_aswindow( asw, dest, True );
 		}
 		if( dvx != 0 || dvy != 0 )
-			on_window_status_changed( asw, True, True );
+			on_window_status_changed( asw, True );
     }
 	display_progress( False, ".");
     return True;
@@ -426,7 +428,7 @@ LOCAL_DEBUG_CALLER_OUT( "new(%d%+d%+d), old(%d%+d%+d), max(%+d,%+d)", new_desk,
                 	if( circ_list[i] != NULL && circ_list[i]->magic == MAGIC_ASWINDOW )
                     	if( ASWIN_DESK(circ_list[i]) == new_desk )
                     	{
-                        	focus_aswindow( circ_list[i] );
+                        	focus_aswindow( circ_list[i], FOCUS_ASW_CAN_AUTORAISE );
                         	break;
                     	}
         	}else
@@ -438,7 +440,8 @@ LOCAL_DEBUG_CALLER_OUT( "new(%d%+d%+d), old(%d%+d%+d), max(%+d,%+d)", new_desk,
 
     	if( IsValidDesk(new_desk) )
 		{
-        	restack_window_list( new_desk, False );
+			apply_stacking_order( new_desk );
+			send_stacking_order( new_desk );
 			FlushAllQueues();              /* yield to modules */
 		}
     	/* Change the look to this desktop's one if it really changed */
@@ -946,7 +949,8 @@ start_background_xfer( ASImage *new_im )
 		/* under Windows XImage must not exceed 32K , or performance drops sagnificantly */
 		data->lines_per_iteration = 7800/new_im->width ;
 #else		
-		data->lines_per_iteration = Scr.MyDisplayHeight/90;
+		data->lines_per_iteration = min(new_im->height,Scr.MyDisplayHeight/
+					(Scr.Feel.desk_cover_animation_steps==0?1:Scr.Feel.desk_cover_animation_steps));
 #endif
 	}
 	if( data->lines_per_iteration == 0 ) 
@@ -957,12 +961,14 @@ start_background_xfer( ASImage *new_im )
 #ifdef XSHMIMAGE		
 	if(check_shmem_images_enabled())
 	{	
-		data->shm_ximage = create_visual_ximage( Scr.asv, new_im->width, data->lines_per_iteration, 0 );
+		data->shm_ximage = create_visual_ximage( Scr.asv, new_im->width, data->lines_per_iteration, DefaultDepth(dpy,DefaultScreen(dpy)) );
 		data->shmseg = ximage2shmseg(data->shm_ximage) ;
 	}
 #endif	   
 
-	do_background_xfer_iter( data );
+	timer_new (100, do_background_xfer_iter, data);	
+
+//	do_background_xfer_iter( data );
 	return data;
 }
 
@@ -993,13 +999,13 @@ LOCAL_DEBUG_CALLER_OUT( "%p:\"%s\", pmap %lX ", vdata, data->im_name, data->targ
 				y = (data->total_lines - (data->lines_done-data->lines_per_iteration)/2) - lines ;
 												   
 			if( xim == NULL ) 
-				xim = create_visual_scratch_ximage( Scr.asv, im->width, lines, 0 );
+				xim = create_visual_scratch_ximage( Scr.asv, im->width, lines, DefaultDepth(dpy,DefaultScreen(dpy)) );
 			LOCAL_DEBUG_OUT( "making ximage %p, starting at %d, and including %d lines", xim, data->lines_done, lines );
 			if( subimage2ximage (Scr.asv, im, 0, y, xim)	)
 			{	
 				LOCAL_DEBUG_OUT( "done, copying to pixmap at %d, (bytes_per_line = %d)", data->lines_done, xim->bytes_per_line );
 				success = put_ximage( Scr.asv, xim, data->target_pmap, 
-					           			Scr.DrawGC,  0, 0, 0, y, im->width, lines );	
+					           			Scr.RootGC,  0, 0, 0, y, im->width, lines );	
 				LOCAL_DEBUG_OUT( "%s", success?"Success":"Failure" );
 				data->lines_done += lines ;
 				++(data->step);
@@ -1089,6 +1095,8 @@ LOCAL_DEBUG_CALLER_OUT( "desk(%d)->old_desk(%d)->new_back(%p)->old_back(%p)", de
     }
 	display_progress( True, "Releasing old background ..."); 
     release_old_background( old_desk, (desk==old_desk) );
+	display_progress( False, "Done."); 
+
 	if( new_back->loaded_pixmap ) 
 	{
 		ASBackgroundHandler *bh = Scr.RootBackground ;			
@@ -1112,6 +1120,7 @@ LOCAL_DEBUG_CALLER_OUT( "desk(%d)->old_desk(%d)->new_back(%p)->old_back(%p)", de
 			display_progress( True, "Setting background to previously generated pixmap ..."); 
 			XSetWindowBackgroundPixmap( dpy, Scr.Root, new_back->loaded_pixmap );
 			XClearWindow( dpy, Scr.Root );
+			display_progress( False, "Done."); 
 			set_xrootpmap_id (Scr.wmprops, new_back->loaded_pixmap );
 			set_as_background(Scr.wmprops, new_back->loaded_pixmap );
 			display_progress( True, "Done with background change. It may take a moment for X server to display it."); 
@@ -1121,7 +1130,7 @@ LOCAL_DEBUG_CALLER_OUT( "desk(%d)->old_desk(%d)->new_back(%p)->old_back(%p)", de
 		}
 	}		  
 
-   	display_progress( True, "Changing background for desktop #%d ...", desk);
+   	display_progress( True, "Changing background for desktop #%d :", desk);
 	
 
     if( new_back->type == MB_BackCmd )
@@ -1130,7 +1139,7 @@ LOCAL_DEBUG_CALLER_OUT( "desk(%d)->old_desk(%d)->new_back(%p)->old_back(%p)", de
     }else
         new_im = make_desktop_image( desk, new_back );
 
-	display_progress( True, "Background image generated."); 
+	display_progress( True, "     Background image generated."); 
     LOCAL_DEBUG_OUT( "im(%p)", new_im );
     if( new_im )
     {
@@ -1171,7 +1180,7 @@ LOCAL_DEBUG_CALLER_OUT( "desk(%d)->old_desk(%d)->new_back(%p)->old_back(%p)", de
 			bh->pmap = None ;
 		if( bh->pmap == None )
 		{
-            bh->pmap = create_visual_pixmap( Scr.asv, Scr.Root, new_im->width, new_im->height, 0 );
+            bh->pmap = create_visual_pixmap( Scr.asv, Scr.Root, new_im->width, new_im->height, DefaultDepth(dpy,DefaultScreen(dpy)));
             LOCAL_DEBUG_OUT( "new root pixmap created with id %lX and size %dx%d", bh->pmap, new_im->width, new_im->height );
 		}else
 		{	
@@ -1213,7 +1222,8 @@ LOCAL_DEBUG_CALLER_OUT( "desk(%d)->old_desk(%d)->new_back(%p)->old_back(%p)", de
 			/* can't animate if pixmap is tiled - X is slow then */
 			!get_flags( Scr.Feel.flags, DontAnimateBackground) && 
 			/* when we do this for the first time  - we better do it all at once */
-			Scr.wmprops->root_pixmap != None )
+			Scr.wmprops->root_pixmap != None 
+		   )
 		{	
 			Bool tiled = ( new_im->width < Scr.MyDisplayWidth || new_im->height < Scr.MyDisplayHeight );
 			if( old_pmap != bh->pmap )
@@ -1227,19 +1237,34 @@ LOCAL_DEBUG_CALLER_OUT( "desk(%d)->old_desk(%d)->new_back(%p)->old_back(%p)", de
 			}
 			if( !already_transferring )
 			{	
-		 		display_progress( True, "Animating background change for better responsiveness ...");
+		 		display_progress( True, "     Animating background change for better responsiveness ...");
+//			    remove_desktop_cover();
 				start_background_xfer( new_im );  /* we need to do it in small steps! */
+				display_progress( False, "Done."); 
 			}
 		}else
 		{	
+			int pid = -1 ;
 			if( !already_transferring )
 			{	
-        		if( !asimage2drawable( Scr.asv, bh->pmap, new_im, Scr.DrawGC, 0, 0, 0, 0, new_im->width, new_im->height, True) )
+				XImage *xim = create_visual_scratch_ximage( Scr.asv, new_im->width, new_im->height, DefaultDepth(dpy,DefaultScreen(dpy)) );
+				Bool success = False ; 
+				if( subimage2ximage (Scr.asv, new_im, 0, 0, xim)	)
+					success = put_ximage( Scr.asv, xim, bh->pmap, Scr.RootGC, 0, 0, 0, 0, new_im->width, new_im->height );	
+				
+				if( !success ) 
 					show_warning( "failed to draw root background onto pixmap");
+
+				XDestroyImage( xim );				   
         		flush_asimage_cache(new_im);
 			}
-        	XSetWindowBackgroundPixmap( dpy, Scr.Root, bh->pmap );
-        	XClearWindow( dpy, Scr.Root );
+			if( pid <= 0 )
+			{
+        		XSetWindowBackgroundPixmap( dpy, Scr.Root, bh->pmap );
+        		XClearWindow( dpy, Scr.Root );
+				if( pid == 0 ) 
+					_exit(0);
+			}
         	set_xrootpmap_id (Scr.wmprops, bh->pmap );
 			set_as_background(Scr.wmprops, bh->pmap );
 		}  
@@ -1256,7 +1281,9 @@ LOCAL_DEBUG_CALLER_OUT( "desk(%d)->old_desk(%d)->new_back(%p)->old_back(%p)", de
 	spool_unfreed_mem (__FUNCTION__, "");
 #endif	
     LOCAL_DEBUG_OUT("done%s","" );
-    remove_desktop_cover();
+	display_progress( True, "Done changing background."); 
+
+//    remove_desktop_cover();
 }
 
 /*************************************************************************
