@@ -74,7 +74,7 @@ get_free_rectangles_iter_func(void *data, void *aux_data)
     {
         int x, y;
         unsigned int width, height, bw;
-        if( ASWIN_GET_FLAGS(asw, AS_Fullscreen ) )
+        if( ASWIN_GET_FLAGS(asw, AS_Fullscreen|AS_ShortLived ) )
 			return True;
 
 		if( !ASWIN_HFLAGS(asw, AS_AvoidCover) && fr_data->min_layer > ASWIN_LAYER(asw))
@@ -213,9 +213,9 @@ make_desktop_grid(int desk, int min_layer, Bool frame_only, int vx, int vy, ASWi
 //    add_canvas_grid( grid_data.grid, Scr.RootCanvas, resist, attract, vx, vy );
 
 	add_gridline( &(grid_data.grid->h_lines), 0,                   0, Scr.MyDisplayWidth,  resist, attract );
-    add_gridline( &(grid_data.grid->h_lines), Scr.MyDisplayHeight, 0, Scr.MyDisplayWidth,  resist, attract );
+    add_gridline( &(grid_data.grid->h_lines), Scr.MyDisplayHeight, 0, Scr.MyDisplayWidth,  attract, resist );
     add_gridline( &(grid_data.grid->v_lines), 0,                   0, Scr.MyDisplayHeight, resist, attract );
-    add_gridline( &(grid_data.grid->v_lines), Scr.MyDisplayWidth , 0, Scr.MyDisplayHeight, resist, attract );
+    add_gridline( &(grid_data.grid->v_lines), Scr.MyDisplayWidth , 0, Scr.MyDisplayHeight, attract, resist );
 
     /* add all the window edges for this desktop : */
     iterate_asbidirlist( Scr.Windows->clients, get_aswindow_grid_iter_func, (void*)&grid_data, NULL, False );
@@ -227,6 +227,25 @@ make_desktop_grid(int desk, int min_layer, Bool frame_only, int vx, int vy, ASWi
     return grid_data.grid;
 }
 
+void setup_aswindow_moveresize(ASWindow *asw,  struct ASMoveResizeData *mvrdata )
+{
+	if( asw->frame_data && asw->tbar )
+	{
+		if( asw->frame_data->condense_titlebar != NO_ALIGN /* && mvrdata->move_only  */)  
+		{
+			if( ASWIN_HFLAGS(asw, AS_VerticalTitle ) )
+				mvrdata->title_west = asw->tbar->width ;
+			else
+				mvrdata->title_north = asw->tbar->height ;
+		}
+	}
+	raise_scren_panframes( ASDefaultScr );
+	mvrdata->below_sibling = get_lowest_panframe(ASDefaultScr);
+	set_moveresize_restrains( mvrdata, asw->hints, asw->status);
+	mvrdata->grid = make_desktop_grid(Scr.CurrentDesk, AS_LayerDesktop, False, Scr.Vx, Scr.Vy, asw);
+	Scr.moveresize_in_progress = mvrdata ;
+}
+
 void apply_aswindow_moveresize(struct ASMoveResizeData *data)
 {
     ASWindow *asw = window2ASWindow( AS_WIDGET_WINDOW(data->mr));
@@ -235,15 +254,58 @@ LOCAL_DEBUG_OUT( "%dx%d%+d%+d", data->curr.width, data->curr.height, data->curr.
     {
 		int new_width = data->curr.width ;
 		int new_height = data->curr.height ;
+		Bool server_grabbed = False ; 
         if( ASWIN_GET_FLAGS( asw, AS_Shaded ) )
 		{
 			new_width = asw->status->width ;
 			new_height = asw->status->height ;
+#if 0
+			/* lets only move us as we are in shaded state : */
+	        move_canvas(  asw->frame_canvas, data->curr.x, data->curr.y );
+		}else
+		{
+			if( data->curr.width != data->last.width || 
+			    data->curr.height != data->last.height )
+			{
+				int client_width = data->curr.width - (asw->frame_canvas->width - asw->client_canvas->width) ; 
+				int client_height = data->curr.height - (asw->frame_canvas->height - asw->client_canvas->height) ; 
+				XGrabServer(dpy);
+				server_grabbed = True ;
+				resize_canvas( asw->client_canvas, client_width, client_height);
+			}
+			moveresize_canvas(  asw->frame_canvas, data->curr.x, data->curr.y, data->curr.width, data->curr.height );
+			ASSync(False);
+#endif		
+		}
+		if( ASWIN_GET_FLAGS( asw, AS_ShapedDecor|AS_Shaped ) && 
+			(data->curr.width > data->last.width || data->curr.height > data->last.height) )
+		{/* this greately reduces flickering on resizing of shaped windows : */
+		    XRectangle    rect;
+    		rect.x = 0;
+    		rect.y = 0;
+			moveresize_canvas(  asw->frame_canvas, data->curr.x, data->curr.y, data->curr.width, data->curr.height );
+			if( data->curr.width > data->last.width ) 
+			{
+				if( get_flags(asw->frame_data->condense_titlebar, ALIGN_LEFT) && asw->tbar )
+					rect.y = asw->tbar->height ; 
+	    		rect.width  = data->curr.width - data->last.width;
+    			rect.height = (int)data->last.height - rect.y;
+				XShapeCombineRectangles ( dpy, asw->frame, ShapeBounding,
+                                		  data->last.width, 0, &rect, 1, ShapeUnion, Unsorted);
+			}
+			if( data->curr.height > data->last.height ) 
+			{
+				if( get_flags(asw->frame_data->condense_titlebar, ALIGN_RIGHT) && asw->tbar )
+					rect.x = asw->tbar->width ; 
+	    		rect.width  = (int)data->curr.width - rect.x;
+    			rect.height = data->curr.height - data->last.height;
+				XShapeCombineRectangles ( dpy, asw->frame, ShapeBounding,
+                                		  0, data->last.height, &rect, 1, ShapeUnion, Unsorted);
+			}
 		}
-		/* lets only move us as we maybe in shaded state : */
-        move_canvas(  asw->frame_canvas, data->curr.x, data->curr.y );
-		ASSync(False);
         moveresize_aswindow_wm( asw, data->curr.x, data->curr.y, new_width, new_height, False);
+		if( server_grabbed ) 
+			XUngrabServer(dpy);
     }
 }
 
@@ -283,7 +345,6 @@ void complete_aswindow_moveresize(struct ASMoveResizeData *data, Bool cancelled)
 		asw->frame_canvas->height = 1;
 		
 		on_window_moveresize( asw, asw->frame );
-/*        SendConfigureNotify(asw); */
         broadcast_config (M_CONFIGURE_WINDOW, asw);
     }
     Scr.moveresize_in_progress = NULL ;
@@ -305,6 +366,7 @@ void complete_aswindow_move(struct ASMoveResizeData *data, Bool cancelled)
             LOCAL_DEBUG_OUT( "%dx%d%+d%+d", data->start.width, data->start.height, data->curr.x, data->curr.y);
             moveresize_aswindow_wm( asw, data->curr.x, data->curr.y, data->start.width, data->start.height, False );
         }
+		
         ASWIN_CLEAR_FLAGS( asw, AS_MoveresizeInProgress );
 		asw->frame_canvas->root_x = -10000 ; 
 		asw->frame_canvas->root_y = -10000;
@@ -344,7 +406,7 @@ apply_placement_result( ASStatusHints *status, XRectangle *anchor, ASHints *hint
     if( get_flags( flags, HeightValue ) && height > 0 )
         status->height = height ;
 
-    status2anchor( anchor, hints, status, Scr.VxMax, Scr.VyMax);
+    status2anchor( anchor, hints, status, Scr.VxMax+Scr.MyDisplayWidth, Scr.VyMax+Scr.MyDisplayHeight);
 }
 
 static int
@@ -1026,6 +1088,7 @@ static Bool do_manual_placement( ASWindow *asw, ASWindowBox *aswbox, ASGeometry
  */
     if( asw->status->width*asw->status->height < (Scr.Feel.OpaqueMove*Scr.MyDisplayWidth*Scr.MyDisplayHeight) / 100 )
     {
+		LOCAL_DEBUG_OUT( "Mapping client window %lX", asw->client_canvas->w );
         map_canvas_window( asw->client_canvas, True );
         map_canvas_window( asw->frame_canvas, False );
 		if( get_desktop_cover_window() != None ) 
@@ -1047,11 +1110,7 @@ static Bool do_manual_placement( ASWindow *asw, ASWindowBox *aswbox, ASGeometry
                                         complete_aswindow_move );
     if( mvrdata )
     {
-		raise_scren_panframes( ASDefaultScr );
-        mvrdata->below_sibling = get_lowest_panframe(ASDefaultScr);
-        set_moveresize_restrains( mvrdata, asw->hints, asw->status);
-        mvrdata->grid = make_desktop_grid( ASWIN_DESK(asw), ASWIN_LAYER(asw), False, Scr.Vx, Scr.Vy, asw );
-        Scr.moveresize_in_progress = mvrdata ;
+		setup_aswindow_moveresize(asw, mvrdata );
         InteractiveMoveLoop ();
     }else
         ASWIN_CLEAR_FLAGS( asw, AS_MoveresizeInProgress );
@@ -1147,11 +1206,11 @@ place_aswindow_in_windowbox( ASWindow *asw, ASWindowBox *aswbox, ASUsePlacementS
                 return False;
         }
         if( area.width <= 0 )
-            area.width = Scr.MyDisplayWidth ;
+            area.width = (Scr.VxMax + Scr.MyDisplayWidth)- area.x ;
         else if( area.x + area.width > Scr.VxMax + Scr.MyDisplayWidth )
             area.width = Scr.VxMax + Scr.MyDisplayWidth - area.x ;
         if( area.height <= 0 )
-            area.height = Scr.MyDisplayHeight ;
+            area.height = (Scr.VyMax + Scr.MyDisplayHeight)- area.y ;
         else if( area.y + area.height > Scr.VyMax + Scr.MyDisplayHeight )
             area.height = Scr.VyMax + Scr.MyDisplayHeight - area.y ;
     }
@@ -1359,7 +1418,7 @@ avoid_covering_aswin_iter_func(void *data, void *aux_data)
     return True;
 }
 
-void enforce_avoid_cover(ASWindow *asw )
+void do_enforce_avoid_cover(ASWindow *asw )
 {
 	if( asw && ASWIN_HFLAGS( asw, AS_AvoidCover|AS_ShortLived ) == AS_AvoidCover )
 	{
@@ -1395,6 +1454,30 @@ void enforce_avoid_cover(ASWindow *asw )
 	}
 }
 
+void
+delayed_enforce_avoid_cover( void* vdata )
+{
+	ASWindow *asw = (ASWindow*)vdata;
+	if( asw && asw->magic == MAGIC_ASWINDOW ) 
+		do_enforce_avoid_cover( asw );
+}
+
+void
+enforce_avoid_cover( ASWindow *asw )
+{
+	/* we do not want to enforce avoid cover right away - clients may want to reposition 
+	   themselves automatically */
+
+	if( asw ) 
+		if( ASWIN_HFLAGS(asw, AS_AvoidCover|AS_ShortLived) == AS_AvoidCover )
+		{
+/* we don't want to remove by  data as there are other functions associated wit this
+*			while( timer_remove_by_data( (void*)asw ) ); */
+			timer_new (500, delayed_enforce_avoid_cover, (void*)asw);	
+		}
+}
+
+
 void obey_avoid_cover(ASWindow *asw, ASStatusHints *tmp_status, XRectangle *tmp_anchor )
 {
     if( asw )
