@@ -268,7 +268,10 @@ LOCAL_DEBUG_OUT( "@@ANIM to(%d)->from(%d)->delta(%d)->step(%d)", to_size, from_s
 			/* when we shade the window - focus gets set to frame window -
 			   we need to revert it back to the client : */
 			if( Scr.Windows->focused == asw )
+			{
 				focus_window( asw, asw->w );
+				autoraise_window( asw );
+			}
             if( !ASWIN_GET_FLAGS(asw, AS_Dead) )
                 XRaiseWindow( dpy, asw->w );
             return 0;
@@ -475,11 +478,12 @@ move_resize_frame_bars( ASWindow *asw, int side, ASOrientation *od, unsigned int
     return rendered;
 }
 
-static void
+static ASFlagType
 resize_frame_subwindows( ASWindow *asw, ASOrientation *od, unsigned int frame_win_width, unsigned int frame_win_height ) 
 {
     register unsigned int *frame_size = &(asw->status->frame_size[0]) ;
     unsigned int normal_width, normal_height ;
+	ASFlagType client_changes = 0;
 
 	if( od == NULL ) 
 		od = get_orientation_data(asw);
@@ -491,11 +495,11 @@ resize_frame_subwindows( ASWindow *asw, ASOrientation *od, unsigned int frame_wi
 				
 	resize_canvases( asw, od, normal_width, normal_height, frame_size );
     if( !ASWIN_GET_FLAGS(asw, AS_Shaded ) )  /* leave shaded client alone ! */
-    	moveresize_canvas( asw->client_canvas,
-                           frame_size[FR_W],
-                           frame_size[FR_N],
-                           (int)frame_win_width-(int)(frame_size[FR_W]+frame_size[FR_E]),
-                           (int)frame_win_height-(int)(frame_size[FR_N]+frame_size[FR_S]));
+    	client_changes = moveresize_canvas( asw->client_canvas,
+                         					frame_size[FR_W], frame_size[FR_N],
+				                           (int)frame_win_width-(int)(frame_size[FR_W]+frame_size[FR_E]),
+                				           (int)frame_win_height-(int)(frame_size[FR_N]+frame_size[FR_S]));
+	return client_changes;
 }
 
 
@@ -504,6 +508,8 @@ Bool
 apply_window_status_size(register ASWindow *asw, ASOrientation *od)
 {
 	Bool moved = False ;
+	Bool resized = False;
+	ASFlagType client_changes = 0;
     /* note that icons are handled by iconbox */
     if( !ASWIN_GET_FLAGS( asw, AS_Iconic ) )
 	{
@@ -531,18 +537,24 @@ LOCAL_DEBUG_OUT( "**CONFG Client(%lx(%s))->status(%ux%u%+d%+d,%s,%s(%d>-%d))",
 				new_height = step_size ;
 			}
         }
-		moved = (	asw->frame_canvas->root_x != asw->status->x ||
-					asw->frame_canvas->root_y != asw->status->y ||
-					asw->frame_canvas->width != new_width ||
+		resized = (	asw->frame_canvas->width != new_width ||
 					asw->frame_canvas->height != new_height );
-
-		if( step_size <= 0 )
-			resize_frame_subwindows( asw, od, new_width, new_height ); 
+		moved = (	asw->frame_canvas->root_x != asw->status->x ||
+					asw->frame_canvas->root_y != asw->status->y );
 
         moveresize_canvas(  asw->frame_canvas, asw->status->x, asw->status->y,
                             new_width, new_height );
+		/* when we resize the client - our frame should already be positioned correctly ! */
+		if( step_size <= 0 )
+			client_changes = resize_frame_subwindows( asw, od, new_width, new_height ); 
+#if 0
+fprintf( stderr, "client_changes = %X, moved = %d. Called from :\n", client_changes, moved);
+print_simple_backtrace();
+#endif
+		if (moved && client_changes == 0)
+			send_canvas_configure_notify (asw->frame_canvas, asw->client_canvas);
     }
-	return moved;
+	return moved || resized || client_changes != 0;
 }
 
 static void
@@ -689,6 +701,7 @@ on_window_moveresize( ASWindow *asw, Window w )
     int i ;
     ASOrientation *od ;
     unsigned int normal_width, normal_height ;
+	Bool update_shape = False ; 
 
 LOCAL_DEBUG_CALLER_OUT( "(%p,%lx,asw->w=%lx)", asw, w, asw->w );
     if( AS_ASSERT(asw) )
@@ -705,14 +718,20 @@ LOCAL_DEBUG_CALLER_OUT( "(%p,%lx,asw->w=%lx)", asw, w, asw->w );
         if( asw->internal && asw->internal->on_moveresize )
             asw->internal->on_moveresize( asw->internal, w );
 
-        if( changes != 0 )
-        {
-            if( ASWIN_GET_FLAGS( asw, AS_Shaped|AS_ShapedDecor ) )
-                SetShape( asw, 0 );
-			else if( get_flags( asw->internal_flags, ASWF_PendingShapeRemoval ) )
-				ClearShape( asw );
-        }
-    }else if( w == asw->frame )
+		update_shape = (changes != 0 );
+		if( XPending(dpy) > 0 ) 
+		{
+			XEvent tmp ; 
+			XNextEvent(dpy, &tmp );
+			if( tmp.type == ConfigureNotify && tmp.xconfigure.window == asw->frame ) 
+				w = asw->frame ; 
+			else
+				XPutBackEvent( dpy, &tmp );
+		}		
+		
+    } 
+	
+	if( w == asw->frame )
     {/* resize canvases here :*/
         int changes = handle_canvas_config (asw->frame_canvas);
 LOCAL_DEBUG_OUT( "changes=0x%X", changes );
@@ -763,6 +782,13 @@ LOCAL_DEBUG_OUT( "changes=0x%X", changes );
 				}
 
 			}
+			if( asw->shading_steps ==  0 )
+			{	
+		        for( i = 0 ; i < FRAME_SIDES ; ++i )
+        		    if( asw->frame_sides[i] )
+						check_frame_side_config( asw, asw->frame_sides[i]->w, od );       
+			}
+			update_shape = True ; 
         }else if( get_flags( changes, CANVAS_MOVED ) )
         {
 			LOCAL_DEBUG_OUT( "window is moved but not resized %s", "");
@@ -774,20 +800,6 @@ LOCAL_DEBUG_OUT( "changes=0x%X", changes );
         if( changes != 0 )
         {
 			LOCAL_DEBUG_OUT( "resized = %X, shaped = %lX", get_flags( changes, CANVAS_RESIZED ), ASWIN_GET_FLAGS( asw, AS_ShapedDecor|AS_Shaped ) );
-            if( get_flags( changes, CANVAS_RESIZED ) )
-			{
-				if( asw->shading_steps ==  0 )
-				{	
-		        	for( i = 0 ; i < FRAME_SIDES ; ++i )
-        		    	if( asw->frame_sides[i] )
-							check_frame_side_config( asw, asw->frame_sides[i]->w, od );       
-				}
-				
-				if( ASWIN_GET_FLAGS( asw, AS_ShapedDecor|AS_Shaped ))
-                	SetShape( asw, 0 );
-				else if( get_flags( asw->internal_flags, ASWF_PendingShapeRemoval ) )
-					ClearShape( asw );
-			} 
 				
 			if( !check_window_offscreen(asw) )
             	update_window_transparency( asw, False );
@@ -807,30 +819,50 @@ LOCAL_DEBUG_OUT( "changes=0x%X", changes );
                 title_size = calculate_astbar_height( asw->icon_title );
                 move_astbar( asw->icon_title, asw->icon_canvas, 0, asw->icon_canvas->height - title_size );
                 set_astbar_size( asw->icon_title, asw->icon_canvas->width, title_size );
-                render_astbar( asw->icon_title, asw->icon_canvas );
+				if( ASWIN_GET_FLAGS(asw, AS_Iconic ) )
+				{
+                	render_astbar( asw->icon_title, asw->icon_canvas );
+				}
                 LOCAL_DEBUG_OUT( "title_size = %d", title_size );
             }
             set_astbar_size( asw->icon_button, asw->icon_canvas->width, asw->icon_canvas->height-title_size );
-            render_astbar( asw->icon_button, asw->icon_canvas );
-            update_canvas_display( asw->icon_canvas );
+		    if( ASWIN_GET_FLAGS(asw, AS_Iconic ) )
+            {
+            	render_astbar( asw->icon_button, asw->icon_canvas );
+            	update_canvas_display( asw->icon_canvas );
+            }
         }
-        broadcast_config (M_CONFIGURE_WINDOW, asw);
+		if( ASWIN_GET_FLAGS(asw, AS_Iconic ) )
+	        broadcast_config (M_CONFIGURE_WINDOW, asw);
     }else if( asw->icon_title_canvas && w == asw->icon_title_canvas->w )
     {
         if( handle_canvas_config(asw->icon_title_canvas) && asw->icon_title )
         {
             LOCAL_DEBUG_OUT( "icon_title resized to %dx%d%+d%+d", asw->icon_title_canvas->width, asw->icon_title_canvas->height, asw->icon_title_canvas->root_x, asw->icon_title_canvas->root_y );
             set_astbar_size( asw->icon_title, asw->icon_title_canvas->width, asw->icon_title->height );
-            render_astbar( asw->icon_title, asw->icon_title_canvas );
-            update_canvas_display( asw->icon_title_canvas );
+			if( ASWIN_GET_FLAGS(asw, AS_Iconic ) )
+			{
+            	render_astbar( asw->icon_title, asw->icon_title_canvas );
+	            update_canvas_display( asw->icon_title_canvas );
+			}
         }
-        broadcast_config (M_CONFIGURE_WINDOW, asw);
+		if( ASWIN_GET_FLAGS(asw, AS_Iconic ) )
+	        broadcast_config (M_CONFIGURE_WINDOW, asw);
     }else if( asw->shading_steps ==  0 ) /* one of the frame canvases :*/
     {
         if( !check_frame_side_config( asw, w, od ) )
             if( asw->internal && asw->internal->on_moveresize )
                 asw->internal->on_moveresize( asw->internal, w );
     }
+
+    if( update_shape )
+	{
+		if( ASWIN_GET_FLAGS( asw, AS_ShapedDecor|AS_Shaped ))
+            SetShape( asw, 0 );
+		else if( get_flags( asw->internal_flags, ASWF_PendingShapeRemoval ) )
+			ClearShape( asw );
+	} 
+
     ASSync(False);
 }
 
@@ -872,10 +904,14 @@ on_window_title_changed( ASWindow *asw, Bool update_display )
 	if( asw->icon_title )
     {
         if( change_astbar_first_label( asw->icon_title, ASWIN_ICON_NAME(asw), ASWIN_ICON_NAME_ENC(asw) ) )
-            on_icon_changed( asw );
+		    if( ASWIN_GET_FLAGS(asw, AS_Iconic ) )
+	            on_icon_changed( asw );
     }
 }
 
+Bool
+collect_aswindow_hints( ASWindow *asw, ASRawHints *raw_hints );
+
 void
 on_window_hints_changed( ASWindow *asw )
 {
@@ -890,10 +926,8 @@ on_window_hints_changed( ASWindow *asw )
         return ;
     if( ASWIN_GET_FLAGS( asw, AS_Dead ) )
 		return;
-    if( !collect_hints( ASDefaultScr, asw->w, HINT_ANY, &raw_hints ) )
+    if( !collect_aswindow_hints( asw, &raw_hints ) )
         return ;
-    if( is_output_level_under_threshold(OUTPUT_LEVEL_HINTS) )
-        print_hints( NULL, NULL, &raw_hints );
 
 	memset( &scratch_status, 0x00, sizeof( scratch_status ) );
 			
@@ -952,7 +986,7 @@ on_window_hints_changed( ASWindow *asw )
 		status_changed = True ;
 	
 	if( status_changed) 
-	    on_window_status_changed( asw, False, True );
+	    on_window_status_changed( asw, True );
 
 	if( mystrcmp( old_hints->res_name, hints->res_name ) != 0 || 
 		mystrcmp( old_hints->res_class, hints->res_class ) != 0)
@@ -1068,7 +1102,7 @@ update_window_tbar_size(	ASWindow *asw )
 
 
 void
-on_window_status_changed( ASWindow *asw, Bool update_display, Bool reconfigured )
+on_window_status_changed( ASWindow *asw, Bool reconfigured )
 {
     char *unfocus_mystyle = NULL ;
     char *frame_unfocus_mystyle = NULL ;
@@ -1081,7 +1115,7 @@ on_window_status_changed( ASWindow *asw, Bool update_display, Bool reconfigured
 
 /*	get_extwm_state_flags (asw->w, &i); */
 	
-LOCAL_DEBUG_CALLER_OUT( "(%p,%s Update display,%s Reconfigured)", asw, update_display?"":"Don't", reconfigured?"":"Not" );
+LOCAL_DEBUG_CALLER_OUT( "(%p,%s Reconfigured)", asw, reconfigured?"":"Not" );
 LOCAL_DEBUG_OUT( "status geometry = %dx%d%+d%+d", asw->status->width, asw->status->height, asw->status->x, asw->status->y );
     if( ASWIN_GET_FLAGS(asw, AS_Iconic ) )
     {
@@ -1090,43 +1124,48 @@ LOCAL_DEBUG_OUT( "status geometry = %dx%d%+d%+d", asw->status->width, asw->statu
                             AS_ICON_TITLE_UNFOCUS_MYSTYLE ;
         if( asw->icon_title )
             changed = set_astbar_style( asw->icon_title, BAR_STATE_UNFOCUSED, unfocus_mystyle );
-        if( changed ) /* now we need to update icon title size */
+        if( changed || reconfigured ) /* now we need to update icon title size */
             on_icon_changed( asw );
     }else
     {
         Bool decor_shaped = False ;
+		MyFrame *frame_data = asw->frame_data ; 
         int back_type;
         ASFlagType *frame_bevel, title_bevel ;
-        int title_cm, title_hue, title_sat ;
+        int title_cm, title_hue = -1, title_sat = -1;
 
         if( ASWIN_GET_FLAGS(asw, AS_Sticky ) )
         {
             back_type = BACK_STICKY ;
-            frame_bevel = &(asw->frame_data->part_sbevel[0]);
-            title_bevel = asw->frame_data->title_sbevel;
-            title_cm    = asw->frame_data->title_scm ;
-			title_hue   = asw->frame_data->title_shue ;
-			title_sat   = asw->frame_data->title_ssat ;
+            frame_bevel = &(frame_data->part_sbevel[0]);
+            title_bevel = frame_data->title_sbevel;
+            title_cm    = frame_data->title_scm ;
+			if( get_flags( frame_data->set_title_attr, MYFRAME_TitleSHueSet )  ) 
+				title_hue   = frame_data->title_shue ;
+			if( get_flags( frame_data->set_title_attr, MYFRAME_TitleSSatSet )  ) 
+				title_sat   = frame_data->title_ssat ;
         }else
         {
             back_type = BACK_UNFOCUSED ;
-            frame_bevel = &(asw->frame_data->part_ubevel[0]);
-            title_bevel = asw->frame_data->title_ubevel;
-            title_cm    = asw->frame_data->title_ucm;
-			title_hue   = asw->frame_data->title_uhue ;
-			title_sat   = asw->frame_data->title_usat ;
+            frame_bevel = &(frame_data->part_ubevel[0]);
+            title_bevel = frame_data->title_ubevel;
+            title_cm    = frame_data->title_ucm;
+			if( get_flags( frame_data->set_title_attr, MYFRAME_TitleUHueSet )  ) 
+				title_hue   = frame_data->title_uhue ;
+			if( get_flags( frame_data->set_title_attr, MYFRAME_TitleUSatSet )  ) 
+				title_sat   = frame_data->title_usat ;
         }
 
         unfocus_mystyle = asw->hints->mystyle_names[back_type];
-        if( asw->frame_data->title_style_names[back_type] )
-            unfocus_mystyle = asw->frame_data->title_style_names[back_type] ;
+        if( frame_data->title_style_names[back_type] )
+            unfocus_mystyle = frame_data->title_style_names[back_type] ;
         if( unfocus_mystyle == NULL )
             unfocus_mystyle = Scr.Look.MSWindow[back_type]->name ;
         if( unfocus_mystyle == NULL )
             unfocus_mystyle = Scr.Look.MSWindow[BACK_UNFOCUSED]->name ;
 
-        frame_unfocus_mystyle = (asw->frame_data->frame_style_names[back_type]==NULL)?
-                                        unfocus_mystyle : asw->frame_data->frame_style_names[back_type];
+        frame_unfocus_mystyle = (frame_data->frame_style_names[back_type]==NULL)?
+                                        unfocus_mystyle : frame_data->frame_style_names[back_type];
 
         for( i = 0 ; i < FRAME_PARTS ; ++i )
 		{
@@ -1474,9 +1513,9 @@ init_aswindow_status( ASWindow *t, ASStatusHints *status )
 			int width = t->status->width ;
 			int height = t->status->height ;
 			if( (!get_flags (t->status->flags, AS_StartSizeUser) && width < t->hints->min_width) || width == 1 )
-				width = min( t->hints->min_width, Scr.MyDisplayWidth );
+				width = min( t->hints->min_width, Scr.VxMax+Scr.MyDisplayWidth );
 			if( (!get_flags (t->status->flags, AS_StartSizeUser) && height < t->hints->min_height) || height == 1 )
-				width = min( t->hints->min_height, Scr.MyDisplayHeight );
+				height = min( t->hints->min_height, Scr.VyMax+Scr.MyDisplayHeight );
 			if( width != t->status->width || height != t->status->height ) 
 			{
 				int dx = 0, dy = 0 ;
@@ -1581,7 +1620,7 @@ init_aswindow_status( ASWindow *t, ASStatusHints *status )
     /* by now we have a valid position for the window: */
     set_flags( t->status->flags, AS_Position );
 
-    status2anchor( &(t->anchor), t->hints, t->status, Scr.VxMax, Scr.VyMax);
+    status2anchor( &(t->anchor), t->hints, t->status, Scr.VxMax+Scr.MyDisplayWidth, Scr.VyMax+Scr.MyDisplayHeight);
     LOCAL_DEBUG_OUT( "status->geom=%dx%d%+d%+d,status->viewport=%+d%+d,anchor=%dx%d%+d%+d",
                      t->status->width, t->status->height, t->status->x, t->status->y,
                      t->status->viewport_x, t->status->viewport_y,
@@ -1684,7 +1723,7 @@ LOCAL_DEBUG_OUT( "unmaping client window 0x%lX", (unsigned long)asw->w );
         map_canvas_window(asw->icon_canvas, True );
         if( asw->icon_canvas != asw->icon_title_canvas )
             map_canvas_window(asw->icon_title_canvas, True );
-        on_window_status_changed( asw, False, True );
+        on_window_status_changed( asw, True );
 LOCAL_DEBUG_OUT( "updating status to iconic for client %p(\"%s\")", asw, ASWIN_NAME(asw) );
     }else
     {   /* Performing transition IconicState->NormalState  */
@@ -1724,7 +1763,7 @@ LOCAL_DEBUG_OUT( "updating status to iconic for client %p(\"%s\")", asw, ASWIN_N
                 }
             }
         }
-	    on_window_status_changed( asw, True, False );
+	    on_window_status_changed( asw, False );
     }
 
     if( !get_flags(asw->wm_state_transition, ASWT_FROM_WITHDRAWN ) )
@@ -1882,31 +1921,28 @@ change_aswindow_layer( ASWindow *asw, int layer )
 {
     if( AS_ASSERT(asw) )
         return;
-    if( ASWIN_LAYER(asw) != layer )
+    if( asw->transient_owner == NULL && ASWIN_LAYER(asw) != layer )
     {
         remove_aswindow_from_layer( asw, ASWIN_LAYER(asw));
 LOCAL_DEBUG_OUT( "changing window's layer to %d", layer );
         ASWIN_LAYER(asw) = layer ;
         add_aswindow_to_layer( asw, layer );
-        restack_window_list( ASWIN_DESK(asw), False );
+        restack_window_list( ASWIN_DESK(asw));
         ASWIN_SET_FLAGS(asw, AS_Layer);
         set_client_state (asw->w, asw->status);
     }
 }
 
-void change_aswindow_desktop( ASWindow *asw, int new_desk, Bool force )
+static void 
+do_change_aswindow_desktop( ASWindow *asw, int new_desk, Bool force )
 {
-    int old_desk ;
-    if( AS_ASSERT(asw) )
-        return ;
-	if( ASWIN_GET_FLAGS(asw, AS_Sticky) ) 
-		new_desk = Scr.CurrentDesk ;
-    
+    int old_desk = ASWIN_DESK(asw) ;
+
 	if( !force && ASWIN_DESK(asw) == new_desk )
         return ;
 
-    old_desk = ASWIN_DESK(asw) ;
     ASWIN_DESK(asw) = new_desk ;
+
     if( !ASWIN_GET_FLAGS(asw, AS_Dead) )
     {
         set_client_desktop( asw->w, new_desk );
@@ -1921,6 +1957,79 @@ void change_aswindow_desktop( ASWindow *asw, int new_desk, Bool force )
     }
 }
 
+static void 
+change_aswindow_desktop_nontransient( ASWindow *asw, int new_desk, Bool force )
+{
+	ASWindow **sublist; 
+	int i ; 
+
+	if( ASWIN_GET_FLAGS(asw, AS_Sticky) ) 
+		new_desk = Scr.CurrentDesk ;
+    
+	do_change_aswindow_desktop( asw, new_desk, force );
+	if( asw->transients ) 
+	{
+		sublist = PVECTOR_HEAD(ASWindow*,asw->transients);
+		for( i = 0 ; i < PVECTOR_USED(asw->transients) ; ++i ) 
+			do_change_aswindow_desktop( sublist[i], new_desk, force );
+	}
+}
+
+struct ChangeGroupDesktopAuxData
+{
+	Window group_lead ;
+	ASWindow *initiator ; 
+	int new_desk ; 
+	Bool force ; 
+	
+};
+
+static Bool
+change_aswindow_desktop_for_group_func(void *data, void *aux_data)
+{
+    ASWindow *asw = (ASWindow*)data;
+	struct ChangeGroupDesktopAuxData *ad = (struct ChangeGroupDesktopAuxData*)aux_data ;
+LOCAL_DEBUG_OUT( "asw = %p(w = %lX), initiator = %p, to = %p, asw->gl = %lX, gl = %lX", 
+				 asw, asw->w, ad->initiator, asw->transient_owner, asw->hints->group_lead, 
+				 ad->group_lead );
+    if( asw && asw != ad->initiator && 
+		asw->transient_owner == NULL && asw->hints->group_lead == ad->group_lead )
+		change_aswindow_desktop_nontransient( asw, ad->new_desk, ad->force );
+    return True;
+}
+
+
+void change_aswindow_desktop( ASWindow *asw, int new_desk, Bool force )
+{
+	ASWindow **sublist; 
+	int i ; 
+
+    if( AS_ASSERT(asw) )
+        return ;
+	if( asw->transient_owner )
+		asw = asw->transient_owner ;
+	
+	change_aswindow_desktop_nontransient( asw, new_desk, force );
+
+	LOCAL_DEBUG_OUT( "group_members = %p; group_lead = %lX", 
+					asw->group_members, asw->hints->group_lead );
+	if( asw->group_members ) 
+	{
+		sublist = PVECTOR_HEAD(ASWindow*,asw->group_members);
+		for( i = 0 ; i < PVECTOR_USED(asw->group_members) ; ++i ) 
+			if( sublist[i]->transient_owner == NULL ) 
+				do_change_aswindow_desktop( sublist[i], new_desk, force );
+	}else if( asw->hints->group_lead != None )
+	{/* sometimes group lead may be unmapped untracked window */
+		struct ChangeGroupDesktopAuxData ad ;
+		ad.group_lead = asw->hints->group_lead ;
+		ad.initiator = asw; 
+		ad.new_desk = new_desk; 
+		ad.force = force ; 
+        iterate_asbidirlist( Scr.Windows->clients, change_aswindow_desktop_for_group_func, &ad, NULL, False );
+	}
+}
+
 static Bool
 restore_anchor_x( ASWindow *asw )
 {
@@ -2025,7 +2134,7 @@ LOCAL_DEBUG_OUT( "flags = %lx, on_flags = %lx, off_flags = %lx", flags, on_flags
 			scratch_status.height = Scr.MyDisplayHeight ;
 	 		
 			save_aswindow_anchor( asw, True, True );		   
-			status2anchor(&(asw->anchor), asw->hints, &scratch_status, Scr.VxMax, Scr.VyMax);
+			status2anchor(&(asw->anchor), asw->hints, &scratch_status, Scr.VxMax+Scr.MyDisplayWidth, Scr.VyMax+Scr.MyDisplayHeight);
 			reconfigured = True ;
 		}
 	}
@@ -2051,7 +2160,7 @@ LOCAL_DEBUG_OUT( "flags = %lx, on_flags = %lx, off_flags = %lx", flags, on_flags
     if( need_placement )
         place_aswindow( asw );
 
-    on_window_status_changed( asw, True, reconfigured );
+    on_window_status_changed( asw, reconfigured );
     if( get_flags( flags, AS_Sticky))
 		update_window_transparency( asw, False );
 	LOCAL_DEBUG_OUT( "Window is %sticky", ASWIN_GET_FLAGS(asw,AS_Sticky)?"S":"NotS");
@@ -2071,8 +2180,11 @@ LOCAL_DEBUG_OUT("current focused is %p, active is %p", Scr.Windows->focused, Scr
         GrabEm (ASDefaultScr, Scr.Feel.cursors[ASCUR_Select]);     /* to prevent Enter Notify events to
                                                       be sent to us while shifting windows around */
         if( (res = make_aswindow_visible( asw, deiconify )) )
-            Scr.Windows->active = asw ;   /* must do that prior to UngrabEm, so that window gets focused */
-        UngrabEm ();
+		{
+			LOCAL_DEBUG_OUT( "CHANGE Scr.Windows->active from %p to %p", Scr.Windows->active, asw );
+	        Scr.Windows->active = asw ;   /* must do that prior to UngrabEm, so that window gets focused */
+        }
+		UngrabEm ();
     }else
     {
         if( ASWIN_GET_FLAGS( asw, AS_Iconic ) )
@@ -2094,6 +2206,7 @@ LOCAL_DEBUG_OUT( "Window is on inactive desk - can't focus%s","");
 LOCAL_DEBUG_OUT( "Window is out of the screen - can't focus%s","");
             return False;                      /* we are out of screen - can't focus */
         }
+		LOCAL_DEBUG_OUT( "CHANGE Scr.Windows->active from %p to %p", Scr.Windows->active, asw );
         Scr.Windows->active = asw ;   /* must do that prior to UngrabEm, so that window gets focused */
         res = focus_active_window();
     }
